"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum
from polar_sdk.types import BaseModel
from polar_sdk.utils import validate_const
import pydantic
from pydantic.functional_validators import AfterValidator
from typing_extensions import Annotated, TypedDict


class ProductPriceOneTimeFreeCreateType(str, Enum):
    ONE_TIME = "one_time"


class ProductPriceOneTimeFreeCreateAmountType(str, Enum):
    FREE = "free"


class ProductPriceOneTimeFreeCreateTypedDict(TypedDict):
    r"""Schema to create a free one-time product price."""

    type: ProductPriceOneTimeFreeCreateType
    amount_type: ProductPriceOneTimeFreeCreateAmountType


class ProductPriceOneTimeFreeCreate(BaseModel):
    r"""Schema to create a free one-time product price."""

    TYPE: Annotated[
        Annotated[
            ProductPriceOneTimeFreeCreateType,
            AfterValidator(validate_const(ProductPriceOneTimeFreeCreateType.ONE_TIME)),
        ],
        pydantic.Field(alias="type"),
    ] = ProductPriceOneTimeFreeCreateType.ONE_TIME

    AMOUNT_TYPE: Annotated[
        Annotated[
            ProductPriceOneTimeFreeCreateAmountType,
            AfterValidator(
                validate_const(ProductPriceOneTimeFreeCreateAmountType.FREE)
            ),
        ],
        pydantic.Field(alias="amount_type"),
    ] = ProductPriceOneTimeFreeCreateAmountType.FREE



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .articlevisibility import ArticleVisibility
from .bylineprofile import BylineProfile, BylineProfileTypedDict
from .organization import Organization, OrganizationTypedDict
from datetime import datetime
from polar_sdk.types import BaseModel, Nullable, UNSET_SENTINEL
from pydantic import model_serializer
from typing_extensions import TypedDict


class ArticleTypedDict(TypedDict):
    id: str
    slug: str
    title: str
    body: str
    byline: BylineProfileTypedDict
    visibility: ArticleVisibility
    user_id: Nullable[str]
    organization_id: str
    organization: OrganizationTypedDict
    published_at: Nullable[datetime]
    paid_subscribers_only: Nullable[bool]
    paid_subscribers_only_ends_at: Nullable[datetime]
    is_preview: bool
    is_pinned: bool
    notify_subscribers: Nullable[bool]
    notifications_sent_at: Nullable[datetime]
    email_sent_to_count: Nullable[int]
    og_image_url: Nullable[str]
    og_description: Nullable[str]


class Article(BaseModel):
    id: str

    slug: str

    title: str

    body: str

    byline: BylineProfile

    visibility: ArticleVisibility

    user_id: Nullable[str]

    organization_id: str

    organization: Organization

    published_at: Nullable[datetime]

    paid_subscribers_only: Nullable[bool]

    paid_subscribers_only_ends_at: Nullable[datetime]

    is_preview: bool

    is_pinned: bool

    notify_subscribers: Nullable[bool]

    notifications_sent_at: Nullable[datetime]

    email_sent_to_count: Nullable[int]

    og_image_url: Nullable[str]

    og_description: Nullable[str]

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = []
        nullable_fields = [
            "user_id",
            "published_at",
            "paid_subscribers_only",
            "paid_subscribers_only_ends_at",
            "notify_subscribers",
            "notifications_sent_at",
            "email_sent_to_count",
            "og_image_url",
            "og_description",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in self.model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .discountfixedonceforeverdurationcreate import (
    DiscountFixedOnceForeverDurationCreate,
    DiscountFixedOnceForeverDurationCreateTypedDict,
)
from .discountfixedrepeatdurationcreate import (
    DiscountFixedRepeatDurationCreate,
    DiscountFixedRepeatDurationCreateTypedDict,
)
from .discountpercentageonceforeverdurationcreate import (
    DiscountPercentageOnceForeverDurationCreate,
    DiscountPercentageOnceForeverDurationCreateTypedDict,
)
from .discountpercentagerepeatdurationcreate import (
    DiscountPercentageRepeatDurationCreate,
    DiscountPercentageRepeatDurationCreateTypedDict,
)
from typing import Union
from typing_extensions import TypeAliasType


DiscountCreateTypedDict = TypeAliasType(
    "DiscountCreateTypedDict",
    Union[
        DiscountPercentageOnceForeverDurationCreateTypedDict,
        DiscountFixedOnceForeverDurationCreateTypedDict,
        DiscountPercentageRepeatDurationCreateTypedDict,
        DiscountFixedRepeatDurationCreateTypedDict,
    ],
)


DiscountCreate = TypeAliasType(
    "DiscountCreate",
    Union[
        DiscountPercentageOnceForeverDurationCreate,
        DiscountFixedOnceForeverDurationCreate,
        DiscountPercentageRepeatDurationCreate,
        DiscountFixedRepeatDurationCreate,
    ],
)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from polar_sdk.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class CustomFieldTextPropertiesTypedDict(TypedDict):
    form_label: NotRequired[str]
    form_help_text: NotRequired[str]
    form_placeholder: NotRequired[str]
    textarea: NotRequired[bool]
    min_length: NotRequired[int]
    max_length: NotRequired[int]


class CustomFieldTextProperties(BaseModel):
    form_label: Optional[str] = None

    form_help_text: Optional[str] = None

    form_placeholder: Optional[str] = None

    textarea: Optional[bool] = None

    min_length: Optional[int] = None

    max_length: Optional[int] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum
from polar_sdk.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from pydantic import model_serializer
from typing_extensions import NotRequired, TypedDict


class BenefitGitHubRepositoryCreatePropertiesPermission(str, Enum):
    r"""The permission level to grant. Read more about roles and their permissions on [GitHub documentation](https://docs.github.com/en/organizations/managing-user-access-to-your-organizations-repositories/managing-repository-roles/repository-roles-for-an-organization#permissions-for-each-role)."""

    PULL = "pull"
    TRIAGE = "triage"
    PUSH = "push"
    MAINTAIN = "maintain"
    ADMIN = "admin"


class BenefitGitHubRepositoryCreatePropertiesTypedDict(TypedDict):
    r"""Properties to create a benefit of type `github_repository`."""

    permission: BenefitGitHubRepositoryCreatePropertiesPermission
    r"""The permission level to grant. Read more about roles and their permissions on [GitHub documentation](https://docs.github.com/en/organizations/managing-user-access-to-your-organizations-repositories/managing-repository-roles/repository-roles-for-an-organization#permissions-for-each-role)."""
    repository_id: NotRequired[Nullable[str]]
    repository_owner: NotRequired[Nullable[str]]
    r"""The owner of the repository."""
    repository_name: NotRequired[Nullable[str]]
    r"""The name of the repository."""


class BenefitGitHubRepositoryCreateProperties(BaseModel):
    r"""Properties to create a benefit of type `github_repository`."""

    permission: BenefitGitHubRepositoryCreatePropertiesPermission
    r"""The permission level to grant. Read more about roles and their permissions on [GitHub documentation](https://docs.github.com/en/organizations/managing-user-access-to-your-organizations-repositories/managing-repository-roles/repository-roles-for-an-organization#permissions-for-each-role)."""

    repository_id: OptionalNullable[str] = UNSET

    repository_owner: OptionalNullable[str] = UNSET
    r"""The owner of the repository."""

    repository_name: OptionalNullable[str] = UNSET
    r"""The name of the repository."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["repository_id", "repository_owner", "repository_name"]
        nullable_fields = ["repository_id", "repository_owner", "repository_name"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in self.model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .productpriceonetime import ProductPriceOneTime, ProductPriceOneTimeTypedDict
from .productpricerecurring import ProductPriceRecurring, ProductPriceRecurringTypedDict
from typing import Union
from typing_extensions import TypeAliasType


ProductPriceTypedDict = TypeAliasType(
    "ProductPriceTypedDict",
    Union[ProductPriceRecurringTypedDict, ProductPriceOneTimeTypedDict],
)


ProductPrice = TypeAliasType(
    "ProductPrice", Union[ProductPriceRecurring, ProductPriceOneTime]
)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .benefit import Benefit, BenefitTypedDict
from enum import Enum
from polar_sdk.types import BaseModel
from polar_sdk.utils import validate_const
import pydantic
from pydantic.functional_validators import AfterValidator
from typing_extensions import Annotated, TypedDict


class WebhookBenefitUpdatedPayloadType(str, Enum):
    BENEFIT_UPDATED = "benefit.updated"


class WebhookBenefitUpdatedPayloadTypedDict(TypedDict):
    r"""Sent when a benefit is updated.

    **Discord & Slack support:** Basic
    """

    data: BenefitTypedDict
    type: WebhookBenefitUpdatedPayloadType


class WebhookBenefitUpdatedPayload(BaseModel):
    r"""Sent when a benefit is updated.

    **Discord & Slack support:** Basic
    """

    data: Benefit

    TYPE: Annotated[
        Annotated[
            WebhookBenefitUpdatedPayloadType,
            AfterValidator(
                validate_const(WebhookBenefitUpdatedPayloadType.BENEFIT_UPDATED)
            ),
        ],
        pydantic.Field(alias="type"),
    ] = WebhookBenefitUpdatedPayloadType.BENEFIT_UPDATED



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from polar_sdk.types import BaseModel
from typing_extensions import TypedDict


class BenefitDiscordCreatePropertiesTypedDict(TypedDict):
    r"""Properties to create a benefit of type `discord`."""

    guild_token: str
    role_id: str
    r"""The ID of the Discord role to grant."""


class BenefitDiscordCreateProperties(BaseModel):
    r"""Properties to create a benefit of type `discord`."""

    guild_token: str

    role_id: str
    r"""The ID of the Discord role to grant."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from polar_sdk.types import BaseModel
from typing_extensions import TypedDict


class CurrencyAmountTypedDict(TypedDict):
    currency: str
    r"""Three letter currency code (eg: USD)"""
    amount: int
    r"""Amount in the currencies smallest unit (cents if currency is USD)"""


class CurrencyAmount(BaseModel):
    currency: str
    r"""Three letter currency code (eg: USD)"""

    amount: int
    r"""Amount in the currencies smallest unit (cents if currency is USD)"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .discountfixedonceforeverdurationbase import (
    DiscountFixedOnceForeverDurationBase,
    DiscountFixedOnceForeverDurationBaseTypedDict,
)
from .discountfixedrepeatdurationbase import (
    DiscountFixedRepeatDurationBase,
    DiscountFixedRepeatDurationBaseTypedDict,
)
from .discountpercentageonceforeverdurationbase import (
    DiscountPercentageOnceForeverDurationBase,
    DiscountPercentageOnceForeverDurationBaseTypedDict,
)
from .discountpercentagerepeatdurationbase import (
    DiscountPercentageRepeatDurationBase,
    DiscountPercentageRepeatDurationBaseTypedDict,
)
from .product import Product, ProductTypedDict
from .productpricerecurring import ProductPriceRecurring, ProductPriceRecurringTypedDict
from .subscriptionrecurringinterval import SubscriptionRecurringInterval
from .subscriptionstatus import SubscriptionStatus
from .subscriptionuser import SubscriptionUser, SubscriptionUserTypedDict
from datetime import datetime
from polar_sdk.types import BaseModel, Nullable, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Dict, Optional, Union
from typing_extensions import NotRequired, TypeAliasType, TypedDict


MetadataTypedDict = TypeAliasType("MetadataTypedDict", Union[str, int, bool])


Metadata = TypeAliasType("Metadata", Union[str, int, bool])


class SubscriptionCustomFieldDataTypedDict(TypedDict):
    r"""Key-value object storing custom field values."""


class SubscriptionCustomFieldData(BaseModel):
    r"""Key-value object storing custom field values."""


SubscriptionDiscountTypedDict = TypeAliasType(
    "SubscriptionDiscountTypedDict",
    Union[
        DiscountPercentageOnceForeverDurationBaseTypedDict,
        DiscountFixedOnceForeverDurationBaseTypedDict,
        DiscountPercentageRepeatDurationBaseTypedDict,
        DiscountFixedRepeatDurationBaseTypedDict,
    ],
)


SubscriptionDiscount = TypeAliasType(
    "SubscriptionDiscount",
    Union[
        DiscountPercentageOnceForeverDurationBase,
        DiscountFixedOnceForeverDurationBase,
        DiscountPercentageRepeatDurationBase,
        DiscountFixedRepeatDurationBase,
    ],
)


class SubscriptionTypedDict(TypedDict):
    created_at: datetime
    r"""Creation timestamp of the object."""
    modified_at: Nullable[datetime]
    r"""Last modification timestamp of the object."""
    id: str
    r"""The ID of the object."""
    amount: Nullable[int]
    currency: Nullable[str]
    recurring_interval: SubscriptionRecurringInterval
    status: SubscriptionStatus
    current_period_start: datetime
    current_period_end: Nullable[datetime]
    cancel_at_period_end: bool
    started_at: Nullable[datetime]
    ended_at: Nullable[datetime]
    user_id: str
    product_id: str
    price_id: str
    discount_id: Nullable[str]
    checkout_id: Nullable[str]
    metadata: Dict[str, MetadataTypedDict]
    user: SubscriptionUserTypedDict
    product: ProductTypedDict
    r"""A product."""
    price: ProductPriceRecurringTypedDict
    discount: Nullable[SubscriptionDiscountTypedDict]
    custom_field_data: NotRequired[SubscriptionCustomFieldDataTypedDict]
    r"""Key-value object storing custom field values."""


class Subscription(BaseModel):
    created_at: datetime
    r"""Creation timestamp of the object."""

    modified_at: Nullable[datetime]
    r"""Last modification timestamp of the object."""

    id: str
    r"""The ID of the object."""

    amount: Nullable[int]

    currency: Nullable[str]

    recurring_interval: SubscriptionRecurringInterval

    status: SubscriptionStatus

    current_period_start: datetime

    current_period_end: Nullable[datetime]

    cancel_at_period_end: bool

    started_at: Nullable[datetime]

    ended_at: Nullable[datetime]

    user_id: str

    product_id: str

    price_id: str

    discount_id: Nullable[str]

    checkout_id: Nullable[str]

    metadata: Dict[str, Metadata]

    user: SubscriptionUser

    product: Product
    r"""A product."""

    price: ProductPriceRecurring

    discount: Nullable[SubscriptionDiscount]

    custom_field_data: Optional[SubscriptionCustomFieldData] = None
    r"""Key-value object storing custom field values."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["custom_field_data"]
        nullable_fields = [
            "modified_at",
            "amount",
            "currency",
            "current_period_end",
            "started_at",
            "ended_at",
            "discount_id",
            "checkout_id",
            "discount",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in self.model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .benefitarticles import BenefitArticles, BenefitArticlesTypedDict
from .benefitbase import BenefitBase, BenefitBaseTypedDict
from .organization import Organization, OrganizationTypedDict
from .productmediafileread import ProductMediaFileRead, ProductMediaFileReadTypedDict
from .productprice import ProductPrice, ProductPriceTypedDict
from datetime import datetime
from polar_sdk.types import BaseModel, Nullable, UNSET_SENTINEL
from pydantic import model_serializer
from typing import List, Union
from typing_extensions import TypeAliasType, TypedDict


BenefitsModelTypedDict = TypeAliasType(
    "BenefitsModelTypedDict", Union[BenefitBaseTypedDict, BenefitArticlesTypedDict]
)


BenefitsModel = TypeAliasType("BenefitsModel", Union[BenefitBase, BenefitArticles])


class UserOrderProductTypedDict(TypedDict):
    created_at: datetime
    r"""Creation timestamp of the object."""
    modified_at: Nullable[datetime]
    r"""Last modification timestamp of the object."""
    id: str
    r"""The ID of the product."""
    name: str
    r"""The name of the product."""
    description: Nullable[str]
    r"""The description of the product."""
    is_recurring: bool
    r"""Whether the product is a subscription tier."""
    is_archived: bool
    r"""Whether the product is archived and no longer available."""
    organization_id: str
    r"""The ID of the organization owning the product."""
    prices: List[ProductPriceTypedDict]
    r"""List of prices for this product."""
    benefits: List[BenefitsModelTypedDict]
    r"""List of benefits granted by the product."""
    medias: List[ProductMediaFileReadTypedDict]
    r"""List of medias associated to the product."""
    organization: OrganizationTypedDict


class UserOrderProduct(BaseModel):
    created_at: datetime
    r"""Creation timestamp of the object."""

    modified_at: Nullable[datetime]
    r"""Last modification timestamp of the object."""

    id: str
    r"""The ID of the product."""

    name: str
    r"""The name of the product."""

    description: Nullable[str]
    r"""The description of the product."""

    is_recurring: bool
    r"""Whether the product is a subscription tier."""

    is_archived: bool
    r"""Whether the product is archived and no longer available."""

    organization_id: str
    r"""The ID of the organization owning the product."""

    prices: List[ProductPrice]
    r"""List of prices for this product."""

    benefits: List[BenefitsModel]
    r"""List of benefits granted by the product."""

    medias: List[ProductMediaFileRead]
    r"""List of medias associated to the product."""

    organization: Organization

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = []
        nullable_fields = ["modified_at", "description"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in self.model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from polar_sdk.types import BaseModel
from typing_extensions import TypedDict


class ArticleReceiversTypedDict(TypedDict):
    free_subscribers: int
    premium_subscribers: int
    organization_members: int


class ArticleReceivers(BaseModel):
    free_subscribers: int

    premium_subscribers: int

    organization_members: int



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from polar_sdk.types import BaseModel
from typing_extensions import TypedDict


class AttachedCustomFieldCreateTypedDict(TypedDict):
    r"""Schema to attach a custom field to a resource."""

    custom_field_id: str
    r"""ID of the custom field to attach."""
    required: bool
    r"""Whether the value is required for this custom field."""


class AttachedCustomFieldCreate(BaseModel):
    r"""Schema to attach a custom field to a resource."""

    custom_field_id: str
    r"""ID of the custom field to attach."""

    required: bool
    r"""Whether the value is required for this custom field."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from polar_sdk.types import BaseModel
from polar_sdk.utils import FieldMetadata, QueryParamMetadata
from typing_extensions import Annotated, TypedDict


class ArticlesExportRequestTypedDict(TypedDict):
    organization_id: str


class ArticlesExportRequest(BaseModel):
    organization_id: Annotated[
        str, FieldMetadata(query=QueryParamMetadata(style="form", explode=True))
    ]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from polar_sdk.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class BenefitAdsPropertiesTypedDict(TypedDict):
    r"""Properties for a benefit of type `ads`."""

    image_height: NotRequired[int]
    r"""The height of the displayed ad."""
    image_width: NotRequired[int]
    r"""The width of the displayed ad."""


class BenefitAdsProperties(BaseModel):
    r"""Properties for a benefit of type `ads`."""

    image_height: Optional[int] = 400
    r"""The height of the displayed ad."""

    image_width: Optional[int] = 400
    r"""The width of the displayed ad."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from polar_sdk.types import BaseModel
from typing_extensions import TypedDict


class LicenseKeyDeactivateTypedDict(TypedDict):
    key: str
    organization_id: str
    activation_id: str


class LicenseKeyDeactivate(BaseModel):
    key: str

    organization_id: str

    activation_id: str



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .pledge import Pledge, PledgeTypedDict
from enum import Enum
from polar_sdk.types import BaseModel
from polar_sdk.utils import validate_const
import pydantic
from pydantic.functional_validators import AfterValidator
from typing_extensions import Annotated, TypedDict


class WebhookPledgeUpdatedPayloadType(str, Enum):
    PLEDGE_UPDATED = "pledge.updated"


class WebhookPledgeUpdatedPayloadTypedDict(TypedDict):
    r"""Sent when a pledge is updated.

    **Discord & Slack support:** Basic
    """

    data: PledgeTypedDict
    type: WebhookPledgeUpdatedPayloadType


class WebhookPledgeUpdatedPayload(BaseModel):
    r"""Sent when a pledge is updated.

    **Discord & Slack support:** Basic
    """

    data: Pledge

    TYPE: Annotated[
        Annotated[
            WebhookPledgeUpdatedPayloadType,
            AfterValidator(
                validate_const(WebhookPledgeUpdatedPayloadType.PLEDGE_UPDATED)
            ),
        ],
        pydantic.Field(alias="type"),
    ] = WebhookPledgeUpdatedPayloadType.PLEDGE_UPDATED



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .address import Address, AddressTypedDict
from .discountfixedonceforeverdurationbase import (
    DiscountFixedOnceForeverDurationBase,
    DiscountFixedOnceForeverDurationBaseTypedDict,
)
from .discountfixedrepeatdurationbase import (
    DiscountFixedRepeatDurationBase,
    DiscountFixedRepeatDurationBaseTypedDict,
)
from .discountpercentageonceforeverdurationbase import (
    DiscountPercentageOnceForeverDurationBase,
    DiscountPercentageOnceForeverDurationBaseTypedDict,
)
from .discountpercentagerepeatdurationbase import (
    DiscountPercentageRepeatDurationBase,
    DiscountPercentageRepeatDurationBaseTypedDict,
)
from .orderbillingreason import OrderBillingReason
from .orderproduct import OrderProduct, OrderProductTypedDict
from .ordersubscription import OrderSubscription, OrderSubscriptionTypedDict
from .orderuser import OrderUser, OrderUserTypedDict
from .productprice import ProductPrice, ProductPriceTypedDict
from datetime import datetime
from polar_sdk.types import BaseModel, Nullable, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Dict, Optional, Union
from typing_extensions import NotRequired, TypeAliasType, TypedDict


OrderMetadataTypedDict = TypeAliasType("OrderMetadataTypedDict", Union[str, int, bool])


OrderMetadata = TypeAliasType("OrderMetadata", Union[str, int, bool])


class OrderCustomFieldDataTypedDict(TypedDict):
    r"""Key-value object storing custom field values."""


class OrderCustomFieldData(BaseModel):
    r"""Key-value object storing custom field values."""


OrderDiscountTypedDict = TypeAliasType(
    "OrderDiscountTypedDict",
    Union[
        DiscountPercentageOnceForeverDurationBaseTypedDict,
        DiscountFixedOnceForeverDurationBaseTypedDict,
        DiscountPercentageRepeatDurationBaseTypedDict,
        DiscountFixedRepeatDurationBaseTypedDict,
    ],
)


OrderDiscount = TypeAliasType(
    "OrderDiscount",
    Union[
        DiscountPercentageOnceForeverDurationBase,
        DiscountFixedOnceForeverDurationBase,
        DiscountPercentageRepeatDurationBase,
        DiscountFixedRepeatDurationBase,
    ],
)


class OrderTypedDict(TypedDict):
    created_at: datetime
    r"""Creation timestamp of the object."""
    modified_at: Nullable[datetime]
    r"""Last modification timestamp of the object."""
    id: str
    r"""The ID of the object."""
    metadata: Dict[str, OrderMetadataTypedDict]
    amount: int
    tax_amount: int
    currency: str
    billing_reason: OrderBillingReason
    billing_address: Nullable[AddressTypedDict]
    user_id: str
    product_id: str
    product_price_id: str
    discount_id: Nullable[str]
    subscription_id: Nullable[str]
    checkout_id: Nullable[str]
    user: OrderUserTypedDict
    product: OrderProductTypedDict
    product_price: ProductPriceTypedDict
    discount: Nullable[OrderDiscountTypedDict]
    subscription: Nullable[OrderSubscriptionTypedDict]
    custom_field_data: NotRequired[OrderCustomFieldDataTypedDict]
    r"""Key-value object storing custom field values."""


class Order(BaseModel):
    created_at: datetime
    r"""Creation timestamp of the object."""

    modified_at: Nullable[datetime]
    r"""Last modification timestamp of the object."""

    id: str
    r"""The ID of the object."""

    metadata: Dict[str, OrderMetadata]

    amount: int

    tax_amount: int

    currency: str

    billing_reason: OrderBillingReason

    billing_address: Nullable[Address]

    user_id: str

    product_id: str

    product_price_id: str

    discount_id: Nullable[str]

    subscription_id: Nullable[str]

    checkout_id: Nullable[str]

    user: OrderUser

    product: OrderProduct

    product_price: ProductPrice

    discount: Nullable[OrderDiscount]

    subscription: Nullable[OrderSubscription]

    custom_field_data: Optional[OrderCustomFieldData] = None
    r"""Key-value object storing custom field values."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["custom_field_data"]
        nullable_fields = [
            "modified_at",
            "billing_address",
            "discount_id",
            "subscription_id",
            "checkout_id",
            "discount",
            "subscription",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in self.model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .subscriptionrecurringinterval import SubscriptionRecurringInterval
from enum import Enum
from polar_sdk.types import BaseModel
from polar_sdk.utils import validate_const
import pydantic
from pydantic.functional_validators import AfterValidator
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class ProductPriceRecurringFixedCreateType(str, Enum):
    RECURRING = "recurring"


class ProductPriceRecurringFixedCreateAmountType(str, Enum):
    FIXED = "fixed"


class ProductPriceRecurringFixedCreateTypedDict(TypedDict):
    r"""Schema to create a recurring product price, i.e. a subscription."""

    price_amount: int
    r"""The price in cents."""
    recurring_interval: SubscriptionRecurringInterval
    type: ProductPriceRecurringFixedCreateType
    amount_type: ProductPriceRecurringFixedCreateAmountType
    price_currency: NotRequired[str]
    r"""The currency. Currently, only `usd` is supported."""


class ProductPriceRecurringFixedCreate(BaseModel):
    r"""Schema to create a recurring product price, i.e. a subscription."""

    price_amount: int
    r"""The price in cents."""

    recurring_interval: SubscriptionRecurringInterval

    TYPE: Annotated[
        Annotated[
            ProductPriceRecurringFixedCreateType,
            AfterValidator(
                validate_const(ProductPriceRecurringFixedCreateType.RECURRING)
            ),
        ],
        pydantic.Field(alias="type"),
    ] = ProductPriceRecurringFixedCreateType.RECURRING

    AMOUNT_TYPE: Annotated[
        Annotated[
            ProductPriceRecurringFixedCreateAmountType,
            AfterValidator(
                validate_const(ProductPriceRecurringFixedCreateAmountType.FIXED)
            ),
        ],
        pydantic.Field(alias="amount_type"),
    ] = ProductPriceRecurringFixedCreateAmountType.FIXED

    price_currency: Optional[str] = "usd"
    r"""The currency. Currently, only `usd` is supported."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .oauth2clientconfigurationupdate import (
    OAuth2ClientConfigurationUpdate,
    OAuth2ClientConfigurationUpdateTypedDict,
)
from polar_sdk.types import BaseModel
from polar_sdk.utils import FieldMetadata, PathParamMetadata, RequestMetadata
from typing_extensions import Annotated, TypedDict


class Oauth2ClientsOauth2UpdateClientRequestTypedDict(TypedDict):
    client_id: str
    o_auth2_client_configuration_update: OAuth2ClientConfigurationUpdateTypedDict


class Oauth2ClientsOauth2UpdateClientRequest(BaseModel):
    client_id: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]

    o_auth2_client_configuration_update: Annotated[
        OAuth2ClientConfigurationUpdate,
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .customfieldcreatecheckbox import (
    CustomFieldCreateCheckbox,
    CustomFieldCreateCheckboxTypedDict,
)
from .customfieldcreatedate import CustomFieldCreateDate, CustomFieldCreateDateTypedDict
from .customfieldcreatenumber import (
    CustomFieldCreateNumber,
    CustomFieldCreateNumberTypedDict,
)
from .customfieldcreateselect import (
    CustomFieldCreateSelect,
    CustomFieldCreateSelectTypedDict,
)
from .customfieldcreatetext import CustomFieldCreateText, CustomFieldCreateTextTypedDict
from polar_sdk.utils import get_discriminator
from pydantic import Discriminator, Tag
from typing import Union
from typing_extensions import Annotated, TypeAliasType


CustomFieldCreateTypedDict = TypeAliasType(
    "CustomFieldCreateTypedDict",
    Union[
        CustomFieldCreateTextTypedDict,
        CustomFieldCreateNumberTypedDict,
        CustomFieldCreateDateTypedDict,
        CustomFieldCreateCheckboxTypedDict,
        CustomFieldCreateSelectTypedDict,
    ],
)


CustomFieldCreate = Annotated[
    Union[
        Annotated[CustomFieldCreateCheckbox, Tag("checkbox")],
        Annotated[CustomFieldCreateDate, Tag("date")],
        Annotated[CustomFieldCreateNumber, Tag("number")],
        Annotated[CustomFieldCreateSelect, Tag("select")],
        Annotated[CustomFieldCreateText, Tag("text")],
    ],
    Discriminator(lambda m: get_discriminator(m, "type", "type")),
]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class CheckoutStatus(str, Enum):
    OPEN = "open"
    EXPIRED = "expired"
    CONFIRMED = "confirmed"
    SUCCEEDED = "succeeded"
    FAILED = "failed"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from jsonpath import JSONPath
from polar_sdk import models, utils
from polar_sdk._hooks import HookContext
from polar_sdk.types import BaseModel, OptionalNullable, UNSET
from typing import Any, Dict, List, Optional, Union, cast


class PolarAdvertisements(BaseSDK):
    def list(
        self,
        *,
        page: Optional[int] = 1,
        limit: Optional[int] = 10,
        sorting: OptionalNullable[List[models.UserAdvertisementSortProperty]] = UNSET,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
    ) -> Optional[models.UsersAdvertisementsListResponse]:
        r"""List Advertisements

        List advertisement campaigns.

        :param page: Page number, defaults to 1.
        :param limit: Size of a page, defaults to 10. Maximum is 100.
        :param sorting: Sorting criterion. Several criteria can be used simultaneously and will be applied in order. Add a minus sign `-` before the criteria name to sort by descending order.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url

        request = models.UsersAdvertisementsListRequest(
            page=page,
            limit=limit,
            sorting=sorting,
        )

        req = self.build_request(
            method="GET",
            path="/v1/users/advertisements/",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                operation_id="users:advertisements:list",
                oauth2_scopes=[],
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        def next_func() -> Optional[models.UsersAdvertisementsListResponse]:
            body = utils.unmarshal_json(http_res.text, Dict[Any, Any])
            page = request.page if not request.page is None else 0
            next_page = page + 1

            num_pages = JSONPath("$.pagination.max_page").parse(body)
            if len(num_pages) == 0 or num_pages[0] <= page:
                return None

            if not http_res.text:
                return None
            results = JSONPath("$.items").parse(body)
            if len(results) == 0 or len(results[0]) == 0:
                return None
            limit = request.limit if not request.limit is None else 0
            if len(results[0]) < limit:
                return None

            return self.list(
                page=next_page,
                limit=limit,
                sorting=sorting,
                retries=retries,
            )

        data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.UsersAdvertisementsListResponse(
                result=utils.unmarshal_json(
                    http_res.text,
                    Optional[models.ListResourceUserAdvertisementCampaign],
                ),
                next=next_func,
            )
        if utils.match_response(http_res, "422", "application/json"):
            data = utils.unmarshal_json(http_res.text, models.HTTPValidationErrorData)
            raise models.HTTPValidationError(data=data)
        if utils.match_response(http_res, ["4XX", "5XX"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.SDKError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = utils.stream_to_text(http_res)
        raise models.SDKError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    async def list_async(
        self,
        *,
        page: Optional[int] = 1,
        limit: Optional[int] = 10,
        sorting: OptionalNullable[List[models.UserAdvertisementSortProperty]] = UNSET,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
    ) -> Optional[models.UsersAdvertisementsListResponse]:
        r"""List Advertisements

        List advertisement campaigns.

        :param page: Page number, defaults to 1.
        :param limit: Size of a page, defaults to 10. Maximum is 100.
        :param sorting: Sorting criterion. Several criteria can be used simultaneously and will be applied in order. Add a minus sign `-` before the criteria name to sort by descending order.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url

        request = models.UsersAdvertisementsListRequest(
            page=page,
            limit=limit,
            sorting=sorting,
        )

        req = self.build_request_async(
            method="GET",
            path="/v1/users/advertisements/",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                operation_id="users:advertisements:list",
                oauth2_scopes=[],
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        def next_func() -> Optional[models.UsersAdvertisementsListResponse]:
            body = utils.unmarshal_json(http_res.text, Dict[Any, Any])
            page = request.page if not request.page is None else 0
            next_page = page + 1

            num_pages = JSONPath("$.pagination.max_page").parse(body)
            if len(num_pages) == 0 or num_pages[0] <= page:
                return None

            if not http_res.text:
                return None
            results = JSONPath("$.items").parse(body)
            if len(results) == 0 or len(results[0]) == 0:
                return None
            limit = request.limit if not request.limit is None else 0
            if len(results[0]) < limit:
                return None

            return self.list(
                page=next_page,
                limit=limit,
                sorting=sorting,
                retries=retries,
            )

        data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.UsersAdvertisementsListResponse(
                result=utils.unmarshal_json(
                    http_res.text,
                    Optional[models.ListResourceUserAdvertisementCampaign],
                ),
                next=next_func,
            )
        if utils.match_response(http_res, "422", "application/json"):
            data = utils.unmarshal_json(http_res.text, models.HTTPValidationErrorData)
            raise models.HTTPValidationError(data=data)
        if utils.match_response(http_res, ["4XX", "5XX"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.SDKError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = await utils.stream_to_text_async(http_res)
        raise models.SDKError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    def create(
        self,
        *,
        request: Union[
            models.UserAdvertisementCampaignCreate,
            models.UserAdvertisementCampaignCreateTypedDict,
        ],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
    ) -> Optional[models.UserAdvertisementCampaign]:
        r"""Create Advertisement

        Create an advertisement campaign.

        :param request: The request object to send.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url

        if not isinstance(request, BaseModel):
            request = utils.unmarshal(request, models.UserAdvertisementCampaignCreate)
        request = cast(models.UserAdvertisementCampaignCreate, request)

        req = self.build_request(
            method="POST",
            path="/v1/users/advertisements/",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", models.UserAdvertisementCampaignCreate
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                operation_id="users:advertisements:create",
                oauth2_scopes=[],
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        data: Any = None
        if utils.match_response(http_res, "201", "application/json"):
            return utils.unmarshal_json(
                http_res.text, Optional[models.UserAdvertisementCampaign]
            )
        if utils.match_response(http_res, "422", "application/json"):
            data = utils.unmarshal_json(http_res.text, models.HTTPValidationErrorData)
            raise models.HTTPValidationError(data=data)
        if utils.match_response(http_res, ["4XX", "5XX"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.SDKError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = utils.stream_to_text(http_res)
        raise models.SDKError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    async def create_async(
        self,
        *,
        request: Union[
            models.UserAdvertisementCampaignCreate,
            models.UserAdvertisementCampaignCreateTypedDict,
        ],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
    ) -> Optional[models.UserAdvertisementCampaign]:
        r"""Create Advertisement

        Create an advertisement campaign.

        :param request: The request object to send.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url

        if not isinstance(request, BaseModel):
            request = utils.unmarshal(request, models.UserAdvertisementCampaignCreate)
        request = cast(models.UserAdvertisementCampaignCreate, request)

        req = self.build_request_async(
            method="POST",
            path="/v1/users/advertisements/",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", models.UserAdvertisementCampaignCreate
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                operation_id="users:advertisements:create",
                oauth2_scopes=[],
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        data: Any = None
        if utils.match_response(http_res, "201", "application/json"):
            return utils.unmarshal_json(
                http_res.text, Optional[models.UserAdvertisementCampaign]
            )
        if utils.match_response(http_res, "422", "application/json"):
            data = utils.unmarshal_json(http_res.text, models.HTTPValidationErrorData)
            raise models.HTTPValidationError(data=data)
        if utils.match_response(http_res, ["4XX", "5XX"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.SDKError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = await utils.stream_to_text_async(http_res)
        raise models.SDKError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    def get(
        self,
        *,
        id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
    ) -> Optional[models.UserAdvertisementCampaign]:
        r"""Get Advertisement

        Get an advertisement campaign by ID.

        :param id: The advertisement campaign ID.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url

        request = models.UsersAdvertisementsGetRequest(
            id=id,
        )

        req = self.build_request(
            method="GET",
            path="/v1/users/advertisements/{id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                operation_id="users:advertisements:get",
                oauth2_scopes=[],
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return utils.unmarshal_json(
                http_res.text, Optional[models.UserAdvertisementCampaign]
            )
        if utils.match_response(http_res, "404", "application/json"):
            data = utils.unmarshal_json(http_res.text, models.ResourceNotFoundData)
            raise models.ResourceNotFound(data=data)
        if utils.match_response(http_res, "422", "application/json"):
            data = utils.unmarshal_json(http_res.text, models.HTTPValidationErrorData)
            raise models.HTTPValidationError(data=data)
        if utils.match_response(http_res, ["4XX", "5XX"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.SDKError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = utils.stream_to_text(http_res)
        raise models.SDKError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    async def get_async(
        self,
        *,
        id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
    ) -> Optional[models.UserAdvertisementCampaign]:
        r"""Get Advertisement

        Get an advertisement campaign by ID.

        :param id: The advertisement campaign ID.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url

        request = models.UsersAdvertisementsGetRequest(
            id=id,
        )

        req = self.build_request_async(
            method="GET",
            path="/v1/users/advertisements/{id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                operation_id="users:advertisements:get",
                oauth2_scopes=[],
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return utils.unmarshal_json(
                http_res.text, Optional[models.UserAdvertisementCampaign]
            )
        if utils.match_response(http_res, "404", "application/json"):
            data = utils.unmarshal_json(http_res.text, models.ResourceNotFoundData)
            raise models.ResourceNotFound(data=data)
        if utils.match_response(http_res, "422", "application/json"):
            data = utils.unmarshal_json(http_res.text, models.HTTPValidationErrorData)
            raise models.HTTPValidationError(data=data)
        if utils.match_response(http_res, ["4XX", "5XX"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.SDKError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = await utils.stream_to_text_async(http_res)
        raise models.SDKError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    def update(
        self,
        *,
        id: str,
        user_advertisement_campaign_update: Union[
            models.UserAdvertisementCampaignUpdate,
            models.UserAdvertisementCampaignUpdateTypedDict,
        ],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
    ) -> Optional[models.UserAdvertisementCampaign]:
        r"""Update Advertisement

        Update an advertisement campaign.

        :param id: The advertisement campaign ID.
        :param user_advertisement_campaign_update:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url

        request = models.UsersAdvertisementsUpdateRequest(
            id=id,
            user_advertisement_campaign_update=utils.get_pydantic_model(
                user_advertisement_campaign_update,
                models.UserAdvertisementCampaignUpdate,
            ),
        )

        req = self.build_request(
            method="PATCH",
            path="/v1/users/advertisements/{id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.user_advertisement_campaign_update,
                False,
                False,
                "json",
                models.UserAdvertisementCampaignUpdate,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                operation_id="users:advertisements:update",
                oauth2_scopes=[],
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return utils.unmarshal_json(
                http_res.text, Optional[models.UserAdvertisementCampaign]
            )
        if utils.match_response(http_res, "404", "application/json"):
            data = utils.unmarshal_json(http_res.text, models.ResourceNotFoundData)
            raise models.ResourceNotFound(data=data)
        if utils.match_response(http_res, "422", "application/json"):
            data = utils.unmarshal_json(http_res.text, models.HTTPValidationErrorData)
            raise models.HTTPValidationError(data=data)
        if utils.match_response(http_res, ["4XX", "5XX"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.SDKError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = utils.stream_to_text(http_res)
        raise models.SDKError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    async def update_async(
        self,
        *,
        id: str,
        user_advertisement_campaign_update: Union[
            models.UserAdvertisementCampaignUpdate,
            models.UserAdvertisementCampaignUpdateTypedDict,
        ],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
    ) -> Optional[models.UserAdvertisementCampaign]:
        r"""Update Advertisement

        Update an advertisement campaign.

        :param id: The advertisement campaign ID.
        :param user_advertisement_campaign_update:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url

        request = models.UsersAdvertisementsUpdateRequest(
            id=id,
            user_advertisement_campaign_update=utils.get_pydantic_model(
                user_advertisement_campaign_update,
                models.UserAdvertisementCampaignUpdate,
            ),
        )

        req = self.build_request_async(
            method="PATCH",
            path="/v1/users/advertisements/{id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.user_advertisement_campaign_update,
                False,
                False,
                "json",
                models.UserAdvertisementCampaignUpdate,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                operation_id="users:advertisements:update",
                oauth2_scopes=[],
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return utils.unmarshal_json(
                http_res.text, Optional[models.UserAdvertisementCampaign]
            )
        if utils.match_response(http_res, "404", "application/json"):
            data = utils.unmarshal_json(http_res.text, models.ResourceNotFoundData)
            raise models.ResourceNotFound(data=data)
        if utils.match_response(http_res, "422", "application/json"):
            data = utils.unmarshal_json(http_res.text, models.HTTPValidationErrorData)
            raise models.HTTPValidationError(data=data)
        if utils.match_response(http_res, ["4XX", "5XX"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.SDKError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = await utils.stream_to_text_async(http_res)
        raise models.SDKError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    def delete(
        self,
        *,
        id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
    ) -> Optional[Any]:
        r"""Delete Advertisement

        Delete an advertisement campaign.

        It'll be automatically disabled on all granted benefits.

        :param id: The advertisement campaign ID.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url

        request = models.UsersAdvertisementsDeleteRequest(
            id=id,
        )

        req = self.build_request(
            method="DELETE",
            path="/v1/users/advertisements/{id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                operation_id="users:advertisements:delete",
                oauth2_scopes=[],
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return utils.unmarshal_json(http_res.text, Optional[Any])
        if utils.match_response(http_res, "204", "*"):
            return None
        if utils.match_response(http_res, "404", "application/json"):
            data = utils.unmarshal_json(http_res.text, models.ResourceNotFoundData)
            raise models.ResourceNotFound(data=data)
        if utils.match_response(http_res, "422", "application/json"):
            data = utils.unmarshal_json(http_res.text, models.HTTPValidationErrorData)
            raise models.HTTPValidationError(data=data)
        if utils.match_response(http_res, ["4XX", "5XX"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.SDKError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = utils.stream_to_text(http_res)
        raise models.SDKError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    async def delete_async(
        self,
        *,
        id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
    ) -> Optional[Any]:
        r"""Delete Advertisement

        Delete an advertisement campaign.

        It'll be automatically disabled on all granted benefits.

        :param id: The advertisement campaign ID.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url

        request = models.UsersAdvertisementsDeleteRequest(
            id=id,
        )

        req = self.build_request_async(
            method="DELETE",
            path="/v1/users/advertisements/{id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                operation_id="users:advertisements:delete",
                oauth2_scopes=[],
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return utils.unmarshal_json(http_res.text, Optional[Any])
        if utils.match_response(http_res, "204", "*"):
            return None
        if utils.match_response(http_res, "404", "application/json"):
            data = utils.unmarshal_json(http_res.text, models.ResourceNotFoundData)
            raise models.ResourceNotFound(data=data)
        if utils.match_response(http_res, "422", "application/json"):
            data = utils.unmarshal_json(http_res.text, models.HTTPValidationErrorData)
            raise models.HTTPValidationError(data=data)
        if utils.match_response(http_res, ["4XX", "5XX"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.SDKError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = await utils.stream_to_text_async(http_res)
        raise models.SDKError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    def enable(
        self,
        *,
        id: str,
        user_advertisement_campaign_enable: Union[
            models.UserAdvertisementCampaignEnable,
            models.UserAdvertisementCampaignEnableTypedDict,
        ],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
    ):
        r"""Enable Advertisement

        Enable an advertisement campaign on a granted benefit.

        :param id: The advertisement campaign ID.
        :param user_advertisement_campaign_enable:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url

        request = models.UsersAdvertisementsEnableRequest(
            id=id,
            user_advertisement_campaign_enable=utils.get_pydantic_model(
                user_advertisement_campaign_enable,
                models.UserAdvertisementCampaignEnable,
            ),
        )

        req = self.build_request(
            method="POST",
            path="/v1/users/advertisements/{id}/enable",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.user_advertisement_campaign_enable,
                False,
                False,
                "json",
                models.UserAdvertisementCampaignEnable,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                operation_id="users:advertisements:enable",
                oauth2_scopes=[],
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        data: Any = None
        if utils.match_response(http_res, "204", "*"):
            return
        if utils.match_response(http_res, "404", "application/json"):
            data = utils.unmarshal_json(http_res.text, models.ResourceNotFoundData)
            raise models.ResourceNotFound(data=data)
        if utils.match_response(http_res, "422", "application/json"):
            data = utils.unmarshal_json(http_res.text, models.HTTPValidationErrorData)
            raise models.HTTPValidationError(data=data)
        if utils.match_response(http_res, ["4XX", "5XX"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.SDKError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = utils.stream_to_text(http_res)
        raise models.SDKError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    async def enable_async(
        self,
        *,
        id: str,
        user_advertisement_campaign_enable: Union[
            models.UserAdvertisementCampaignEnable,
            models.UserAdvertisementCampaignEnableTypedDict,
        ],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
    ):
        r"""Enable Advertisement

        Enable an advertisement campaign on a granted benefit.

        :param id: The advertisement campaign ID.
        :param user_advertisement_campaign_enable:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url

        request = models.UsersAdvertisementsEnableRequest(
            id=id,
            user_advertisement_campaign_enable=utils.get_pydantic_model(
                user_advertisement_campaign_enable,
                models.UserAdvertisementCampaignEnable,
            ),
        )

        req = self.build_request_async(
            method="POST",
            path="/v1/users/advertisements/{id}/enable",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.user_advertisement_campaign_enable,
                False,
                False,
                "json",
                models.UserAdvertisementCampaignEnable,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                operation_id="users:advertisements:enable",
                oauth2_scopes=[],
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        data: Any = None
        if utils.match_response(http_res, "204", "*"):
            return
        if utils.match_response(http_res, "404", "application/json"):
            data = utils.unmarshal_json(http_res.text, models.ResourceNotFoundData)
            raise models.ResourceNotFound(data=data)
        if utils.match_response(http_res, "422", "application/json"):
            data = utils.unmarshal_json(http_res.text, models.HTTPValidationErrorData)
            raise models.HTTPValidationError(data=data)
        if utils.match_response(http_res, ["4XX", "5XX"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.SDKError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = await utils.stream_to_text_async(http_res)
        raise models.SDKError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from polar_sdk.types import BaseModel
from typing_extensions import TypedDict


class ArticlePreviewTypedDict(TypedDict):
    email: str
    r"""Email address to send the preview to. The user must be registered on Polar."""


class ArticlePreview(BaseModel):
    email: str
    r"""Email address to send the preview to. The user must be registered on Polar."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from datetime import datetime
from polar_sdk.types import BaseModel
from typing_extensions import TypedDict


class MetricPeriodTypedDict(TypedDict):
    timestamp: datetime
    r"""Timestamp of this period data."""
    orders: int
    revenue: int
    average_order_value: int
    one_time_products: int
    one_time_products_revenue: int
    new_subscriptions: int
    new_subscriptions_revenue: int
    renewed_subscriptions: int
    renewed_subscriptions_revenue: int
    active_subscriptions: int
    monthly_recurring_revenue: int


class MetricPeriod(BaseModel):
    timestamp: datetime
    r"""Timestamp of this period data."""

    orders: int

    revenue: int

    average_order_value: int

    one_time_products: int

    one_time_products_revenue: int

    new_subscriptions: int

    new_subscriptions_revenue: int

    renewed_subscriptions: int

    renewed_subscriptions_revenue: int

    active_subscriptions: int

    monthly_recurring_revenue: int



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum
from polar_sdk import utils
from polar_sdk.types import BaseModel
from polar_sdk.utils import validate_const
import pydantic
from pydantic.functional_validators import AfterValidator
from typing_extensions import Annotated


class Error(str, Enum):
    RESOURCE_NOT_FOUND = "ResourceNotFound"


class ResourceNotFoundData(BaseModel):
    detail: str

    ERROR: Annotated[
        Annotated[Error, AfterValidator(validate_const(Error.RESOURCE_NOT_FOUND))],
        pydantic.Field(alias="error"),
    ] = Error.RESOURCE_NOT_FOUND


class ResourceNotFound(Exception):
    data: ResourceNotFoundData

    def __init__(self, data: ResourceNotFoundData):
        self.data = data

    def __str__(self) -> str:
        return utils.marshal_json(self.data, ResourceNotFoundData)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .customfieldcheckbox import CustomFieldCheckbox, CustomFieldCheckboxTypedDict
from .customfielddate import CustomFieldDate, CustomFieldDateTypedDict
from .customfieldnumber import CustomFieldNumber, CustomFieldNumberTypedDict
from .customfieldselect import CustomFieldSelect, CustomFieldSelectTypedDict
from .customfieldtext import CustomFieldText, CustomFieldTextTypedDict
from polar_sdk.utils import get_discriminator
from pydantic import Discriminator, Tag
from typing import Union
from typing_extensions import Annotated, TypeAliasType


CustomFieldTypedDict = TypeAliasType(
    "CustomFieldTypedDict",
    Union[
        CustomFieldTextTypedDict,
        CustomFieldNumberTypedDict,
        CustomFieldDateTypedDict,
        CustomFieldCheckboxTypedDict,
        CustomFieldSelectTypedDict,
    ],
)


CustomField = Annotated[
    Union[
        Annotated[CustomFieldCheckbox, Tag("checkbox")],
        Annotated[CustomFieldDate, Tag("date")],
        Annotated[CustomFieldNumber, Tag("number")],
        Annotated[CustomFieldSelect, Tag("select")],
        Annotated[CustomFieldText, Tag("text")],
    ],
    Discriminator(lambda m: get_discriminator(m, "type", "type")),
]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .product import Product, ProductTypedDict
from enum import Enum
from polar_sdk.types import BaseModel
from polar_sdk.utils import validate_const
import pydantic
from pydantic.functional_validators import AfterValidator
from typing_extensions import Annotated, TypedDict


class WebhookProductCreatedPayloadType(str, Enum):
    PRODUCT_CREATED = "product.created"


class WebhookProductCreatedPayloadTypedDict(TypedDict):
    r"""Sent when a new product is created.

    **Discord & Slack support:** Basic
    """

    data: ProductTypedDict
    r"""A product."""
    type: WebhookProductCreatedPayloadType


class WebhookProductCreatedPayload(BaseModel):
    r"""Sent when a new product is created.

    **Discord & Slack support:** Basic
    """

    data: Product
    r"""A product."""

    TYPE: Annotated[
        Annotated[
            WebhookProductCreatedPayloadType,
            AfterValidator(
                validate_const(WebhookProductCreatedPayloadType.PRODUCT_CREATED)
            ),
        ],
        pydantic.Field(alias="type"),
    ] = WebhookProductCreatedPayloadType.PRODUCT_CREATED



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .customfieldselectproperties import (
    CustomFieldSelectProperties,
    CustomFieldSelectPropertiesTypedDict,
)
from enum import Enum
from polar_sdk.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from polar_sdk.utils import validate_const
import pydantic
from pydantic import model_serializer
from pydantic.functional_validators import AfterValidator
from typing import Dict, Optional, Union
from typing_extensions import Annotated, NotRequired, TypeAliasType, TypedDict


CustomFieldCreateSelectMetadataTypedDict = TypeAliasType(
    "CustomFieldCreateSelectMetadataTypedDict", Union[str, int, bool]
)


CustomFieldCreateSelectMetadata = TypeAliasType(
    "CustomFieldCreateSelectMetadata", Union[str, int, bool]
)


class CustomFieldCreateSelectType(str, Enum):
    SELECT = "select"


class CustomFieldCreateSelectTypedDict(TypedDict):
    r"""Schema to create a custom field of type select."""

    slug: str
    r"""Identifier of the custom field. It'll be used as key when storing the value. Must be unique across the organization.It can only contain ASCII letters, numbers and hyphens."""
    name: str
    r"""Name of the custom field."""
    properties: CustomFieldSelectPropertiesTypedDict
    metadata: NotRequired[Dict[str, CustomFieldCreateSelectMetadataTypedDict]]
    r"""Key-value object allowing you to store additional information.

    The key must be a string with a maximum length of **40 characters**.
    The value must be either:

    * A string with a maximum length of **500 characters**
    * An integer
    * A boolean

    You can store up to **50 key-value pairs**.
    """
    type: CustomFieldCreateSelectType
    organization_id: NotRequired[Nullable[str]]
    r"""The ID of the organization owning the custom field. **Required unless you use an organization token.**"""


class CustomFieldCreateSelect(BaseModel):
    r"""Schema to create a custom field of type select."""

    slug: str
    r"""Identifier of the custom field. It'll be used as key when storing the value. Must be unique across the organization.It can only contain ASCII letters, numbers and hyphens."""

    name: str
    r"""Name of the custom field."""

    properties: CustomFieldSelectProperties

    metadata: Optional[Dict[str, CustomFieldCreateSelectMetadata]] = None
    r"""Key-value object allowing you to store additional information.

    The key must be a string with a maximum length of **40 characters**.
    The value must be either:

    * A string with a maximum length of **500 characters**
    * An integer
    * A boolean

    You can store up to **50 key-value pairs**.
    """

    TYPE: Annotated[
        Annotated[
            CustomFieldCreateSelectType,
            AfterValidator(validate_const(CustomFieldCreateSelectType.SELECT)),
        ],
        pydantic.Field(alias="type"),
    ] = CustomFieldCreateSelectType.SELECT

    organization_id: OptionalNullable[str] = UNSET
    r"""The ID of the organization owning the custom field. **Required unless you use an organization token.**"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["metadata", "organization_id"]
        nullable_fields = ["organization_id"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in self.model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .subscription import Subscription, SubscriptionTypedDict
from enum import Enum
from polar_sdk.types import BaseModel
from polar_sdk.utils import validate_const
import pydantic
from pydantic.functional_validators import AfterValidator
from typing_extensions import Annotated, TypedDict


class WebhookSubscriptionCreatedPayloadType(str, Enum):
    SUBSCRIPTION_CREATED = "subscription.created"


class WebhookSubscriptionCreatedPayloadTypedDict(TypedDict):
    r"""Sent when a new subscription is created.

    **Discord & Slack support:** Full
    """

    data: SubscriptionTypedDict
    type: WebhookSubscriptionCreatedPayloadType


class WebhookSubscriptionCreatedPayload(BaseModel):
    r"""Sent when a new subscription is created.

    **Discord & Slack support:** Full
    """

    data: Subscription

    TYPE: Annotated[
        Annotated[
            WebhookSubscriptionCreatedPayloadType,
            AfterValidator(
                validate_const(
                    WebhookSubscriptionCreatedPayloadType.SUBSCRIPTION_CREATED
                )
            ),
        ],
        pydantic.Field(alias="type"),
    ] = WebhookSubscriptionCreatedPayloadType.SUBSCRIPTION_CREATED



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum
from polar_sdk.types import BaseModel
from polar_sdk.utils import validate_const
import pydantic
from pydantic.functional_validators import AfterValidator
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class ProductPriceOneTimeFixedCreateType(str, Enum):
    ONE_TIME = "one_time"


class ProductPriceOneTimeFixedCreateAmountType(str, Enum):
    FIXED = "fixed"


class ProductPriceOneTimeFixedCreateTypedDict(TypedDict):
    r"""Schema to create a one-time product price."""

    price_amount: int
    r"""The price in cents."""
    type: ProductPriceOneTimeFixedCreateType
    amount_type: ProductPriceOneTimeFixedCreateAmountType
    price_currency: NotRequired[str]
    r"""The currency. Currently, only `usd` is supported."""


class ProductPriceOneTimeFixedCreate(BaseModel):
    r"""Schema to create a one-time product price."""

    price_amount: int
    r"""The price in cents."""

    TYPE: Annotated[
        Annotated[
            ProductPriceOneTimeFixedCreateType,
            AfterValidator(validate_const(ProductPriceOneTimeFixedCreateType.ONE_TIME)),
        ],
        pydantic.Field(alias="type"),
    ] = ProductPriceOneTimeFixedCreateType.ONE_TIME

    AMOUNT_TYPE: Annotated[
        Annotated[
            ProductPriceOneTimeFixedCreateAmountType,
            AfterValidator(
                validate_const(ProductPriceOneTimeFixedCreateAmountType.FIXED)
            ),
        ],
        pydantic.Field(alias="amount_type"),
    ] = ProductPriceOneTimeFixedCreateAmountType.FIXED

    price_currency: Optional[str] = "usd"
    r"""The currency. Currently, only `usd` is supported."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .pledge import Pledge, PledgeTypedDict
from enum import Enum
from polar_sdk.types import BaseModel
from polar_sdk.utils import validate_const
import pydantic
from pydantic.functional_validators import AfterValidator
from typing_extensions import Annotated, TypedDict


class WebhookPledgeCreatedPayloadType(str, Enum):
    PLEDGE_CREATED = "pledge.created"


class WebhookPledgeCreatedPayloadTypedDict(TypedDict):
    r"""Sent when a new pledge is created. Note that this does mean that the pledge has been paid yet.

    **Discord & Slack support:** Full
    """

    data: PledgeTypedDict
    type: WebhookPledgeCreatedPayloadType


class WebhookPledgeCreatedPayload(BaseModel):
    r"""Sent when a new pledge is created. Note that this does mean that the pledge has been paid yet.

    **Discord & Slack support:** Full
    """

    data: Pledge

    TYPE: Annotated[
        Annotated[
            WebhookPledgeCreatedPayloadType,
            AfterValidator(
                validate_const(WebhookPledgeCreatedPayloadType.PLEDGE_CREATED)
            ),
        ],
        pydantic.Field(alias="type"),
    ] = WebhookPledgeCreatedPayloadType.PLEDGE_CREATED



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .address import Address, AddressTypedDict
from .attachedcustomfield import AttachedCustomField, AttachedCustomFieldTypedDict
from .checkoutdiscountfixedonceforeverduration import (
    CheckoutDiscountFixedOnceForeverDuration,
    CheckoutDiscountFixedOnceForeverDurationTypedDict,
)
from .checkoutdiscountfixedrepeatduration import (
    CheckoutDiscountFixedRepeatDuration,
    CheckoutDiscountFixedRepeatDurationTypedDict,
)
from .checkoutdiscountpercentageonceforeverduration import (
    CheckoutDiscountPercentageOnceForeverDuration,
    CheckoutDiscountPercentageOnceForeverDurationTypedDict,
)
from .checkoutdiscountpercentagerepeatduration import (
    CheckoutDiscountPercentageRepeatDuration,
    CheckoutDiscountPercentageRepeatDurationTypedDict,
)
from .checkoutproduct import CheckoutProduct, CheckoutProductTypedDict
from .checkoutstatus import CheckoutStatus
from .organization import Organization, OrganizationTypedDict
from .paymentprocessor import PaymentProcessor
from .productprice import ProductPrice, ProductPriceTypedDict
from datetime import datetime
from polar_sdk.types import BaseModel, Nullable, UNSET_SENTINEL
from polar_sdk.utils import validate_const
import pydantic
from pydantic import model_serializer
from pydantic.functional_validators import AfterValidator
from typing import List, Optional, Union
from typing_extensions import Annotated, NotRequired, TypeAliasType, TypedDict


class CheckoutPublicCustomFieldDataTypedDict(TypedDict):
    r"""Key-value object storing custom field values."""


class CheckoutPublicCustomFieldData(BaseModel):
    r"""Key-value object storing custom field values."""


class CheckoutPublicPaymentProcessorMetadataTypedDict(TypedDict):
    pass


class CheckoutPublicPaymentProcessorMetadata(BaseModel):
    pass


CheckoutPublicDiscountTypedDict = TypeAliasType(
    "CheckoutPublicDiscountTypedDict",
    Union[
        CheckoutDiscountPercentageOnceForeverDurationTypedDict,
        CheckoutDiscountFixedOnceForeverDurationTypedDict,
        CheckoutDiscountPercentageRepeatDurationTypedDict,
        CheckoutDiscountFixedRepeatDurationTypedDict,
    ],
)


CheckoutPublicDiscount = TypeAliasType(
    "CheckoutPublicDiscount",
    Union[
        CheckoutDiscountPercentageOnceForeverDuration,
        CheckoutDiscountFixedOnceForeverDuration,
        CheckoutDiscountPercentageRepeatDuration,
        CheckoutDiscountFixedRepeatDuration,
    ],
)


class CheckoutPublicTypedDict(TypedDict):
    r"""Checkout session data retrieved using the client secret."""

    created_at: datetime
    r"""Creation timestamp of the object."""
    modified_at: Nullable[datetime]
    r"""Last modification timestamp of the object."""
    id: str
    r"""The ID of the object."""
    status: CheckoutStatus
    client_secret: str
    r"""Client secret used to update and complete the checkout session from the client."""
    url: str
    r"""URL where the customer can access the checkout session."""
    expires_at: datetime
    r"""Expiration date and time of the checkout session."""
    success_url: str
    r"""URL where the customer will be redirected after a successful payment."""
    embed_origin: Nullable[str]
    r"""When checkout is embedded, represents the Origin of the page embedding the checkout. Used as a security measure to send messages only to the embedding page."""
    amount: Nullable[int]
    tax_amount: Nullable[int]
    r"""Computed tax amount to pay in cents."""
    currency: Nullable[str]
    r"""Currency code of the checkout session."""
    subtotal_amount: Nullable[int]
    r"""Subtotal amount in cents, including discounts and before tax."""
    total_amount: Nullable[int]
    r"""Total amount to pay in cents, including discounts and after tax."""
    product_id: str
    r"""ID of the product to checkout."""
    product_price_id: str
    r"""ID of the product price to checkout."""
    discount_id: Nullable[str]
    r"""ID of the discount applied to the checkout."""
    allow_discount_codes: bool
    r"""Whether to allow the customer to apply discount codes. If you apply a discount through `discount_id`, it'll still be applied, but the customer won't be able to change it."""
    is_discount_applicable: bool
    r"""Whether the discount is applicable to the checkout. Typically, free and custom prices are not discountable."""
    is_free_product_price: bool
    r"""Whether the product price is free, regardless of discounts."""
    is_payment_required: bool
    r"""Whether the checkout requires payment, e.g. in case of free products or discounts that cover the total amount."""
    is_payment_setup_required: bool
    r"""Whether the checkout requires setting up a payment method, regardless of the amount, e.g. subscriptions that have first free cycles."""
    is_payment_form_required: bool
    r"""Whether the checkout requires a payment form, whether because of a payment or payment method setup."""
    customer_id: Nullable[str]
    customer_name: Nullable[str]
    customer_email: Nullable[str]
    customer_ip_address: Nullable[str]
    customer_billing_address: Nullable[AddressTypedDict]
    customer_tax_id: Nullable[str]
    payment_processor_metadata: CheckoutPublicPaymentProcessorMetadataTypedDict
    product: CheckoutProductTypedDict
    r"""Product data for a checkout session."""
    product_price: ProductPriceTypedDict
    discount: Nullable[CheckoutPublicDiscountTypedDict]
    organization: OrganizationTypedDict
    attached_custom_fields: List[AttachedCustomFieldTypedDict]
    custom_field_data: NotRequired[CheckoutPublicCustomFieldDataTypedDict]
    r"""Key-value object storing custom field values."""
    payment_processor: PaymentProcessor


class CheckoutPublic(BaseModel):
    r"""Checkout session data retrieved using the client secret."""

    created_at: datetime
    r"""Creation timestamp of the object."""

    modified_at: Nullable[datetime]
    r"""Last modification timestamp of the object."""

    id: str
    r"""The ID of the object."""

    status: CheckoutStatus

    client_secret: str
    r"""Client secret used to update and complete the checkout session from the client."""

    url: str
    r"""URL where the customer can access the checkout session."""

    expires_at: datetime
    r"""Expiration date and time of the checkout session."""

    success_url: str
    r"""URL where the customer will be redirected after a successful payment."""

    embed_origin: Nullable[str]
    r"""When checkout is embedded, represents the Origin of the page embedding the checkout. Used as a security measure to send messages only to the embedding page."""

    amount: Nullable[int]

    tax_amount: Nullable[int]
    r"""Computed tax amount to pay in cents."""

    currency: Nullable[str]
    r"""Currency code of the checkout session."""

    subtotal_amount: Nullable[int]
    r"""Subtotal amount in cents, including discounts and before tax."""

    total_amount: Nullable[int]
    r"""Total amount to pay in cents, including discounts and after tax."""

    product_id: str
    r"""ID of the product to checkout."""

    product_price_id: str
    r"""ID of the product price to checkout."""

    discount_id: Nullable[str]
    r"""ID of the discount applied to the checkout."""

    allow_discount_codes: bool
    r"""Whether to allow the customer to apply discount codes. If you apply a discount through `discount_id`, it'll still be applied, but the customer won't be able to change it."""

    is_discount_applicable: bool
    r"""Whether the discount is applicable to the checkout. Typically, free and custom prices are not discountable."""

    is_free_product_price: bool
    r"""Whether the product price is free, regardless of discounts."""

    is_payment_required: bool
    r"""Whether the checkout requires payment, e.g. in case of free products or discounts that cover the total amount."""

    is_payment_setup_required: bool
    r"""Whether the checkout requires setting up a payment method, regardless of the amount, e.g. subscriptions that have first free cycles."""

    is_payment_form_required: bool
    r"""Whether the checkout requires a payment form, whether because of a payment or payment method setup."""

    customer_id: Nullable[str]

    customer_name: Nullable[str]

    customer_email: Nullable[str]

    customer_ip_address: Nullable[str]

    customer_billing_address: Nullable[Address]

    customer_tax_id: Nullable[str]

    payment_processor_metadata: CheckoutPublicPaymentProcessorMetadata

    product: CheckoutProduct
    r"""Product data for a checkout session."""

    product_price: ProductPrice

    discount: Nullable[CheckoutPublicDiscount]

    organization: Organization

    attached_custom_fields: List[AttachedCustomField]

    custom_field_data: Optional[CheckoutPublicCustomFieldData] = None
    r"""Key-value object storing custom field values."""

    PAYMENT_PROCESSOR: Annotated[
        Annotated[
            PaymentProcessor, AfterValidator(validate_const(PaymentProcessor.STRIPE))
        ],
        pydantic.Field(alias="payment_processor"),
    ] = PaymentProcessor.STRIPE

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["custom_field_data"]
        nullable_fields = [
            "modified_at",
            "embed_origin",
            "amount",
            "tax_amount",
            "currency",
            "subtotal_amount",
            "total_amount",
            "discount_id",
            "customer_id",
            "customer_name",
            "customer_email",
            "customer_ip_address",
            "customer_billing_address",
            "customer_tax_id",
            "discount",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in self.model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .checkoutlinkpricecreate import (
    CheckoutLinkPriceCreate,
    CheckoutLinkPriceCreateTypedDict,
)
from .checkoutlinkproductcreate import (
    CheckoutLinkProductCreate,
    CheckoutLinkProductCreateTypedDict,
)
from typing import Union
from typing_extensions import TypeAliasType


CheckoutLinkCreateTypedDict = TypeAliasType(
    "CheckoutLinkCreateTypedDict",
    Union[CheckoutLinkProductCreateTypedDict, CheckoutLinkPriceCreateTypedDict],
)


CheckoutLinkCreate = TypeAliasType(
    "CheckoutLinkCreate", Union[CheckoutLinkProductCreate, CheckoutLinkPriceCreate]
)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .benefitadssubscriber import BenefitAdsSubscriber, BenefitAdsSubscriberTypedDict
from .benefitarticlessubscriber import (
    BenefitArticlesSubscriber,
    BenefitArticlesSubscriberTypedDict,
)
from .benefitcustomsubscriber import (
    BenefitCustomSubscriber,
    BenefitCustomSubscriberTypedDict,
)
from .benefitdiscordsubscriber import (
    BenefitDiscordSubscriber,
    BenefitDiscordSubscriberTypedDict,
)
from .benefitdownloadablessubscriber import (
    BenefitDownloadablesSubscriber,
    BenefitDownloadablesSubscriberTypedDict,
)
from .benefitgithubrepositorysubscriber import (
    BenefitGitHubRepositorySubscriber,
    BenefitGitHubRepositorySubscriberTypedDict,
)
from .benefitlicensekeyssubscriber import (
    BenefitLicenseKeysSubscriber,
    BenefitLicenseKeysSubscriberTypedDict,
)
from polar_sdk.types import BaseModel
from polar_sdk.utils import FieldMetadata, PathParamMetadata, get_discriminator
from pydantic import Discriminator, Tag
from typing import Union
from typing_extensions import Annotated, TypeAliasType, TypedDict


class UsersBenefitsGetRequestTypedDict(TypedDict):
    id: str
    r"""The benefit ID."""


class UsersBenefitsGetRequest(BaseModel):
    id: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]
    r"""The benefit ID."""


UsersBenefitsGetResponseUsersBenefitsGetTypedDict = TypeAliasType(
    "UsersBenefitsGetResponseUsersBenefitsGetTypedDict",
    Union[
        BenefitArticlesSubscriberTypedDict,
        BenefitAdsSubscriberTypedDict,
        BenefitDiscordSubscriberTypedDict,
        BenefitCustomSubscriberTypedDict,
        BenefitGitHubRepositorySubscriberTypedDict,
        BenefitDownloadablesSubscriberTypedDict,
        BenefitLicenseKeysSubscriberTypedDict,
    ],
)
r"""Successful Response"""


UsersBenefitsGetResponseUsersBenefitsGet = Annotated[
    Union[
        Annotated[BenefitArticlesSubscriber, Tag("articles")],
        Annotated[BenefitAdsSubscriber, Tag("ads")],
        Annotated[BenefitDiscordSubscriber, Tag("discord")],
        Annotated[BenefitCustomSubscriber, Tag("custom")],
        Annotated[BenefitGitHubRepositorySubscriber, Tag("github_repository")],
        Annotated[BenefitDownloadablesSubscriber, Tag("downloadables")],
        Annotated[BenefitLicenseKeysSubscriber, Tag("license_keys")],
    ],
    Discriminator(lambda m: get_discriminator(m, "type", "type")),
]
r"""Successful Response"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .subscription import Subscription, SubscriptionTypedDict
from enum import Enum
from polar_sdk.types import BaseModel
from polar_sdk.utils import validate_const
import pydantic
from pydantic.functional_validators import AfterValidator
from typing_extensions import Annotated, TypedDict


class WebhookSubscriptionUpdatedPayloadType(str, Enum):
    SUBSCRIPTION_UPDATED = "subscription.updated"


class WebhookSubscriptionUpdatedPayloadTypedDict(TypedDict):
    r"""Sent when a subscription is updated. This event fires for all changes to the subscription, including renewals.

    If you want more specific events, you can listen to `subscription.active`, `subscription.canceled`, and `subscription.revoked`.

    To listen specifically for renewals, you can listen to `order.created` events and check the `billing_reason` field.

    **Discord & Slack support:** On cancellation and revocation. Renewals are skipped.
    """

    data: SubscriptionTypedDict
    type: WebhookSubscriptionUpdatedPayloadType


class WebhookSubscriptionUpdatedPayload(BaseModel):
    r"""Sent when a subscription is updated. This event fires for all changes to the subscription, including renewals.

    If you want more specific events, you can listen to `subscription.active`, `subscription.canceled`, and `subscription.revoked`.

    To listen specifically for renewals, you can listen to `order.created` events and check the `billing_reason` field.

    **Discord & Slack support:** On cancellation and revocation. Renewals are skipped.
    """

    data: Subscription

    TYPE: Annotated[
        Annotated[
            WebhookSubscriptionUpdatedPayloadType,
            AfterValidator(
                validate_const(
                    WebhookSubscriptionUpdatedPayloadType.SUBSCRIPTION_UPDATED
                )
            ),
        ],
        pydantic.Field(alias="type"),
    ] = WebhookSubscriptionUpdatedPayloadType.SUBSCRIPTION_UPDATED



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from polar_sdk.types import BaseModel
from typing_extensions import TypedDict


class ReactionsTypedDict(TypedDict):
    total_count: int
    plus_one: int
    minus_one: int
    laugh: int
    hooray: int
    confused: int
    heart: int
    rocket: int
    eyes: int


class Reactions(BaseModel):
    total_count: int

    plus_one: int

    minus_one: int

    laugh: int

    hooray: int

    confused: int

    heart: int

    rocket: int

    eyes: int



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum
from polar_sdk.types import BaseModel
from typing_extensions import TypedDict


class Timeframe(str, Enum):
    YEAR = "year"
    MONTH = "month"
    DAY = "day"


class BenefitLicenseKeyExpirationPropertiesTypedDict(TypedDict):
    ttl: int
    timeframe: Timeframe


class BenefitLicenseKeyExpirationProperties(BaseModel):
    ttl: int

    timeframe: Timeframe



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class PledgeState(str, Enum):
    INITIATED = "initiated"
    CREATED = "created"
    PENDING = "pending"
    REFUNDED = "refunded"
    DISPUTED = "disputed"
    CHARGE_DISPUTED = "charge_disputed"
    CANCELLED = "cancelled"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .benefitgithubrepositorysubscriberproperties import (
    BenefitGitHubRepositorySubscriberProperties,
    BenefitGitHubRepositorySubscriberPropertiesTypedDict,
)
from .benefitgrantsubscriber import (
    BenefitGrantSubscriber,
    BenefitGrantSubscriberTypedDict,
)
from .organization import Organization, OrganizationTypedDict
from datetime import datetime
from enum import Enum
from polar_sdk.types import BaseModel, Nullable, UNSET_SENTINEL
from polar_sdk.utils import validate_const
import pydantic
from pydantic import model_serializer
from pydantic.functional_validators import AfterValidator
from typing import List
from typing_extensions import Annotated, TypedDict


class BenefitGitHubRepositorySubscriberType(str, Enum):
    GITHUB_REPOSITORY = "github_repository"


class BenefitGitHubRepositorySubscriberTypedDict(TypedDict):
    created_at: datetime
    r"""Creation timestamp of the object."""
    modified_at: Nullable[datetime]
    r"""Last modification timestamp of the object."""
    id: str
    r"""The ID of the benefit."""
    description: str
    r"""The description of the benefit."""
    selectable: bool
    r"""Whether the benefit is selectable when creating a product."""
    deletable: bool
    r"""Whether the benefit is deletable."""
    organization_id: str
    r"""The ID of the organization owning the benefit."""
    grants: List[BenefitGrantSubscriberTypedDict]
    organization: OrganizationTypedDict
    properties: BenefitGitHubRepositorySubscriberPropertiesTypedDict
    r"""Properties available to subscribers for a benefit of type `github_repository`."""
    type: BenefitGitHubRepositorySubscriberType


class BenefitGitHubRepositorySubscriber(BaseModel):
    created_at: datetime
    r"""Creation timestamp of the object."""

    modified_at: Nullable[datetime]
    r"""Last modification timestamp of the object."""

    id: str
    r"""The ID of the benefit."""

    description: str
    r"""The description of the benefit."""

    selectable: bool
    r"""Whether the benefit is selectable when creating a product."""

    deletable: bool
    r"""Whether the benefit is deletable."""

    organization_id: str
    r"""The ID of the organization owning the benefit."""

    grants: List[BenefitGrantSubscriber]

    organization: Organization

    properties: BenefitGitHubRepositorySubscriberProperties
    r"""Properties available to subscribers for a benefit of type `github_repository`."""

    TYPE: Annotated[
        Annotated[
            BenefitGitHubRepositorySubscriberType,
            AfterValidator(
                validate_const(BenefitGitHubRepositorySubscriberType.GITHUB_REPOSITORY)
            ),
        ],
        pydantic.Field(alias="type"),
    ] = BenefitGitHubRepositorySubscriberType.GITHUB_REPOSITORY

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = []
        nullable_fields = ["modified_at"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in self.model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from polar_sdk.types import BaseModel
from typing_extensions import TypedDict


class BenefitArticlesPropertiesTypedDict(TypedDict):
    r"""Properties for a benefit of type `articles`."""

    paid_articles: bool
    r"""Whether the user can access paid articles."""


class BenefitArticlesProperties(BaseModel):
    r"""Properties for a benefit of type `articles`."""

    paid_articles: bool
    r"""Whether the user can access paid articles."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .authorizeresponseorganization import (
    AuthorizeResponseOrganization,
    AuthorizeResponseOrganizationTypedDict,
)
from .authorizeresponseuser import AuthorizeResponseUser, AuthorizeResponseUserTypedDict
from polar_sdk.utils import get_discriminator
from pydantic import Discriminator, Tag
from typing import Union
from typing_extensions import Annotated, TypeAliasType


Oauth2AuthorizeResponseOauth2AuthorizeTypedDict = TypeAliasType(
    "Oauth2AuthorizeResponseOauth2AuthorizeTypedDict",
    Union[AuthorizeResponseUserTypedDict, AuthorizeResponseOrganizationTypedDict],
)
r"""Successful Response"""


Oauth2AuthorizeResponseOauth2Authorize = Annotated[
    Union[
        Annotated[AuthorizeResponseUser, Tag("user")],
        Annotated[AuthorizeResponseOrganization, Tag("organization")],
    ],
    Discriminator(lambda m: get_discriminator(m, "sub_type", "sub_type")),
]
r"""Successful Response"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .order import Order, OrderTypedDict
from enum import Enum
from polar_sdk.types import BaseModel
from polar_sdk.utils import validate_const
import pydantic
from pydantic.functional_validators import AfterValidator
from typing_extensions import Annotated, TypedDict


class WebhookOrderCreatedPayloadType(str, Enum):
    ORDER_CREATED = "order.created"


class WebhookOrderCreatedPayloadTypedDict(TypedDict):
    r"""Sent when a new order is created.

    **Discord & Slack support:** Full
    """

    data: OrderTypedDict
    type: WebhookOrderCreatedPayloadType


class WebhookOrderCreatedPayload(BaseModel):
    r"""Sent when a new order is created.

    **Discord & Slack support:** Full
    """

    data: Order

    TYPE: Annotated[
        Annotated[
            WebhookOrderCreatedPayloadType,
            AfterValidator(
                validate_const(WebhookOrderCreatedPayloadType.ORDER_CREATED)
            ),
        ],
        pydantic.Field(alias="type"),
    ] = WebhookOrderCreatedPayloadType.ORDER_CREATED



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .onev1_1oauth2_1token_post_x_components_authorizationcodetokenrequest import (
    Onev11oauth21tokenPostXComponentsAuthorizationCodeTokenRequest,
    Onev11oauth21tokenPostXComponentsAuthorizationCodeTokenRequestTypedDict,
)
from .onev1_1oauth2_1token_post_x_components_refreshtokenrequest import (
    Onev11oauth21tokenPostXComponentsRefreshTokenRequest,
    Onev11oauth21tokenPostXComponentsRefreshTokenRequestTypedDict,
)
from polar_sdk.utils import get_discriminator
from pydantic import Discriminator, Tag
from typing import Union
from typing_extensions import Annotated, TypeAliasType


Oauth2RequestTokenRequestBodyTypedDict = TypeAliasType(
    "Oauth2RequestTokenRequestBodyTypedDict",
    Union[
        Onev11oauth21tokenPostXComponentsRefreshTokenRequestTypedDict,
        Onev11oauth21tokenPostXComponentsAuthorizationCodeTokenRequestTypedDict,
    ],
)


Oauth2RequestTokenRequestBody = Annotated[
    Union[
        Annotated[
            Onev11oauth21tokenPostXComponentsAuthorizationCodeTokenRequest,
            Tag("authorization_code"),
        ],
        Annotated[
            Onev11oauth21tokenPostXComponentsRefreshTokenRequest, Tag("refresh_token")
        ],
    ],
    Discriminator(lambda m: get_discriminator(m, "grant_type", "grant_type")),
]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .pagination import Pagination, PaginationTypedDict
from .useradvertisementcampaign import (
    UserAdvertisementCampaign,
    UserAdvertisementCampaignTypedDict,
)
from polar_sdk.types import BaseModel
from typing import List
from typing_extensions import TypedDict


class ListResourceUserAdvertisementCampaignTypedDict(TypedDict):
    items: List[UserAdvertisementCampaignTypedDict]
    pagination: PaginationTypedDict


class ListResourceUserAdvertisementCampaign(BaseModel):
    items: List[UserAdvertisementCampaign]

    pagination: Pagination



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from datetime import datetime
from enum import Enum
from polar_sdk.types import BaseModel, Nullable, UNSET_SENTINEL
from polar_sdk.utils import validate_const
import pydantic
from pydantic import model_serializer
from pydantic.functional_validators import AfterValidator
from typing_extensions import Annotated, TypedDict


class DownloadableFileReadService(str, Enum):
    DOWNLOADABLE = "downloadable"


class DownloadableFileReadTypedDict(TypedDict):
    r"""File to be associated with the downloadables benefit."""

    id: str
    r"""The ID of the object."""
    organization_id: str
    name: str
    path: str
    mime_type: str
    size: int
    storage_version: Nullable[str]
    checksum_etag: Nullable[str]
    checksum_sha256_base64: Nullable[str]
    checksum_sha256_hex: Nullable[str]
    last_modified_at: Nullable[datetime]
    version: Nullable[str]
    is_uploaded: bool
    created_at: datetime
    size_readable: str
    service: DownloadableFileReadService


class DownloadableFileRead(BaseModel):
    r"""File to be associated with the downloadables benefit."""

    id: str
    r"""The ID of the object."""

    organization_id: str

    name: str

    path: str

    mime_type: str

    size: int

    storage_version: Nullable[str]

    checksum_etag: Nullable[str]

    checksum_sha256_base64: Nullable[str]

    checksum_sha256_hex: Nullable[str]

    last_modified_at: Nullable[datetime]

    version: Nullable[str]

    is_uploaded: bool

    created_at: datetime

    size_readable: str

    SERVICE: Annotated[
        Annotated[
            DownloadableFileReadService,
            AfterValidator(validate_const(DownloadableFileReadService.DOWNLOADABLE)),
        ],
        pydantic.Field(alias="service"),
    ] = DownloadableFileReadService.DOWNLOADABLE

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = []
        nullable_fields = [
            "storage_version",
            "checksum_etag",
            "checksum_sha256_base64",
            "checksum_sha256_hex",
            "last_modified_at",
            "version",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in self.model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum
from polar_sdk.types import BaseModel
from polar_sdk.utils import FieldMetadata, validate_const
import pydantic
from pydantic.functional_validators import AfterValidator
from typing_extensions import Annotated, TypedDict


class Onev11oauth21tokenPostXComponentsRefreshTokenRequestGrantType(str, Enum):
    REFRESH_TOKEN = "refresh_token"


class Onev11oauth21tokenPostXComponentsRefreshTokenRequestTypedDict(TypedDict):
    client_id: str
    client_secret: str
    refresh_token: str
    grant_type: Onev11oauth21tokenPostXComponentsRefreshTokenRequestGrantType


class Onev11oauth21tokenPostXComponentsRefreshTokenRequest(BaseModel):
    client_id: Annotated[str, FieldMetadata(form=True)]

    client_secret: Annotated[str, FieldMetadata(form=True)]

    refresh_token: Annotated[str, FieldMetadata(form=True)]

    GRANT_TYPE: Annotated[
        Annotated[
            Onev11oauth21tokenPostXComponentsRefreshTokenRequestGrantType,
            AfterValidator(
                validate_const(
                    Onev11oauth21tokenPostXComponentsRefreshTokenRequestGrantType.REFRESH_TOKEN
                )
            ),
        ],
        pydantic.Field(alias="grant_type"),
        FieldMetadata(form=True),
    ] = Onev11oauth21tokenPostXComponentsRefreshTokenRequestGrantType.REFRESH_TOKEN



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .s3fileuploadpart import S3FileUploadPart, S3FileUploadPartTypedDict
from polar_sdk.types import BaseModel
from typing import List
from typing_extensions import TypedDict


class S3FileUploadMultipartTypedDict(TypedDict):
    id: str
    path: str
    parts: List[S3FileUploadPartTypedDict]


class S3FileUploadMultipart(BaseModel):
    id: str

    path: str

    parts: List[S3FileUploadPart]



import base64
from typing import Any, Dict, Union

from pydantic import Discriminator, Tag, TypeAdapter
from standardwebhooks.webhooks import Webhook
from standardwebhooks.webhooks import (
    WebhookVerificationError as _WebhookVerificationError,
)
from typing_extensions import Annotated

from polar_sdk.models import (
    WebhookBenefitCreatedPayload,
    WebhookBenefitCreatedPayloadType,
    WebhookBenefitGrantCreatedPayload,
    WebhookBenefitGrantCreatedPayloadType,
    WebhookBenefitGrantRevokedPayload,
    WebhookBenefitGrantRevokedPayloadType,
    WebhookBenefitGrantUpdatedPayload,
    WebhookBenefitGrantUpdatedPayloadType,
    WebhookBenefitUpdatedPayload,
    WebhookBenefitUpdatedPayloadType,
    WebhookCheckoutCreatedPayload,
    WebhookCheckoutCreatedPayloadType,
    WebhookCheckoutUpdatedPayload,
    WebhookCheckoutUpdatedPayloadType,
    WebhookOrderCreatedPayload,
    WebhookOrderCreatedPayloadType,
    WebhookOrganizationUpdatedPayload,
    WebhookOrganizationUpdatedPayloadType,
    WebhookPledgeCreatedPayload,
    WebhookPledgeCreatedPayloadType,
    WebhookPledgeUpdatedPayload,
    WebhookPledgeUpdatedPayloadType,
    WebhookProductCreatedPayload,
    WebhookProductCreatedPayloadType,
    WebhookProductUpdatedPayload,
    WebhookProductUpdatedPayloadType,
    WebhookSubscriptionActivePayload,
    WebhookSubscriptionActivePayloadType,
    WebhookSubscriptionCanceledPayload,
    WebhookSubscriptionCanceledPayloadType,
    WebhookSubscriptionCreatedPayload,
    WebhookSubscriptionCreatedPayloadType,
    WebhookSubscriptionRevokedPayload,
    WebhookSubscriptionRevokedPayloadType,
    WebhookSubscriptionUpdatedPayload,
    WebhookSubscriptionUpdatedPayloadType,
)


def _get_discriminator(v: Any) -> Union[str, None]:
    if isinstance(v, dict):
        return v.get("type")
    return getattr(v, "type", None)


WebhoookPayload = Annotated[
    Union[
        Annotated[
            WebhookBenefitCreatedPayload,
            Tag(WebhookBenefitCreatedPayloadType.BENEFIT_CREATED),
        ],
        Annotated[
            WebhookBenefitGrantCreatedPayload,
            Tag(WebhookBenefitGrantCreatedPayloadType.BENEFIT_GRANT_CREATED),
        ],
        Annotated[
            WebhookBenefitGrantRevokedPayload,
            Tag(WebhookBenefitGrantRevokedPayloadType.BENEFIT_GRANT_REVOKED),
        ],
        Annotated[
            WebhookBenefitGrantUpdatedPayload,
            Tag(WebhookBenefitGrantUpdatedPayloadType.BENEFIT_GRANT_UPDATED),
        ],
        Annotated[
            WebhookBenefitUpdatedPayload,
            Tag(WebhookBenefitUpdatedPayloadType.BENEFIT_UPDATED),
        ],
        Annotated[
            WebhookCheckoutCreatedPayload,
            Tag(WebhookCheckoutCreatedPayloadType.CHECKOUT_CREATED),
        ],
        Annotated[
            WebhookCheckoutUpdatedPayload,
            Tag(WebhookCheckoutUpdatedPayloadType.CHECKOUT_UPDATED),
        ],
        Annotated[
            WebhookOrderCreatedPayload,
            Tag(WebhookOrderCreatedPayloadType.ORDER_CREATED),
        ],
        Annotated[
            WebhookOrganizationUpdatedPayload,
            Tag(WebhookOrganizationUpdatedPayloadType.ORGANIZATION_UPDATED),
        ],
        Annotated[
            WebhookPledgeCreatedPayload,
            Tag(WebhookPledgeCreatedPayloadType.PLEDGE_CREATED),
        ],
        Annotated[
            WebhookPledgeUpdatedPayload,
            Tag(WebhookPledgeUpdatedPayloadType.PLEDGE_UPDATED),
        ],
        Annotated[
            WebhookProductCreatedPayload,
            Tag(WebhookProductCreatedPayloadType.PRODUCT_CREATED),
        ],
        Annotated[
            WebhookProductUpdatedPayload,
            Tag(WebhookProductUpdatedPayloadType.PRODUCT_UPDATED),
        ],
        Annotated[
            WebhookSubscriptionActivePayload,
            Tag(WebhookSubscriptionActivePayloadType.SUBSCRIPTION_ACTIVE),
        ],
        Annotated[
            WebhookSubscriptionCanceledPayload,
            Tag(WebhookSubscriptionCanceledPayloadType.SUBSCRIPTION_CANCELED),
        ],
        Annotated[
            WebhookSubscriptionCreatedPayload,
            Tag(WebhookSubscriptionCreatedPayloadType.SUBSCRIPTION_CREATED),
        ],
        Annotated[
            WebhookSubscriptionRevokedPayload,
            Tag(WebhookSubscriptionRevokedPayloadType.SUBSCRIPTION_REVOKED),
        ],
        Annotated[
            WebhookSubscriptionUpdatedPayload,
            Tag(WebhookSubscriptionUpdatedPayloadType.SUBSCRIPTION_UPDATED),
        ],
    ],
    Discriminator(_get_discriminator),
]

WebhookPayloadAdapter: TypeAdapter[WebhoookPayload] = TypeAdapter(WebhoookPayload)


class WebhookVerificationError(_WebhookVerificationError):
    def __init__(self, message: str) -> None:
        self.message = message
        super().__init__(message)


def validate_event(
    body: Union[str, bytes], headers: Dict[str, str], secret: str
) -> WebhoookPayload:
    base64_secret = base64.b64encode(secret.encode()).decode()
    webhook = Webhook(base64_secret)

    try:
        data = webhook.verify(body, headers)
    except _WebhookVerificationError as e:
        raise WebhookVerificationError(str(e)) from e

    return WebhookPayloadAdapter.validate_python(data)


__all__ = [
    "WebhookVerificationError",
    "WebhoookPayload",
    "validate_event",
]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from polar_sdk.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class CustomFieldDatePropertiesTypedDict(TypedDict):
    form_label: NotRequired[str]
    form_help_text: NotRequired[str]
    form_placeholder: NotRequired[str]
    ge: NotRequired[int]
    le: NotRequired[int]


class CustomFieldDateProperties(BaseModel):
    form_label: Optional[str] = None

    form_help_text: Optional[str] = None

    form_placeholder: Optional[str] = None

    ge: Optional[int] = None

    le: Optional[int] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from polar_sdk.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class OrganizationFeatureSettingsTypedDict(TypedDict):
    articles_enabled: NotRequired[bool]
    r"""If this organization has articles enabled"""
    issue_funding_enabled: NotRequired[bool]
    r"""If this organization has issue funding enabled"""


class OrganizationFeatureSettings(BaseModel):
    articles_enabled: Optional[bool] = False
    r"""If this organization has articles enabled"""

    issue_funding_enabled: Optional[bool] = False
    r"""If this organization has issue funding enabled"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from polar_sdk.types import BaseModel
from typing_extensions import TypedDict


class UserAdvertisementCampaignEnableTypedDict(TypedDict):
    benefit_id: str
    r"""The benefit ID."""


class UserAdvertisementCampaignEnable(BaseModel):
    benefit_id: str
    r"""The benefit ID."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .customfielddateproperties import (
    CustomFieldDateProperties,
    CustomFieldDatePropertiesTypedDict,
)
from datetime import datetime
from enum import Enum
from polar_sdk.types import BaseModel, Nullable, UNSET_SENTINEL
from polar_sdk.utils import validate_const
import pydantic
from pydantic import model_serializer
from pydantic.functional_validators import AfterValidator
from typing import Dict, Union
from typing_extensions import Annotated, TypeAliasType, TypedDict


CustomFieldDateMetadataTypedDict = TypeAliasType(
    "CustomFieldDateMetadataTypedDict", Union[str, int, bool]
)


CustomFieldDateMetadata = TypeAliasType(
    "CustomFieldDateMetadata", Union[str, int, bool]
)


class CustomFieldDateType(str, Enum):
    DATE = "date"


class CustomFieldDateTypedDict(TypedDict):
    r"""Schema for a custom field of type date."""

    created_at: datetime
    r"""Creation timestamp of the object."""
    modified_at: Nullable[datetime]
    r"""Last modification timestamp of the object."""
    id: str
    r"""The ID of the object."""
    metadata: Dict[str, CustomFieldDateMetadataTypedDict]
    slug: str
    r"""Identifier of the custom field. It'll be used as key when storing the value."""
    name: str
    r"""Name of the custom field."""
    organization_id: str
    r"""The ID of the organization owning the custom field."""
    properties: CustomFieldDatePropertiesTypedDict
    type: CustomFieldDateType


class CustomFieldDate(BaseModel):
    r"""Schema for a custom field of type date."""

    created_at: datetime
    r"""Creation timestamp of the object."""

    modified_at: Nullable[datetime]
    r"""Last modification timestamp of the object."""

    id: str
    r"""The ID of the object."""

    metadata: Dict[str, CustomFieldDateMetadata]

    slug: str
    r"""Identifier of the custom field. It'll be used as key when storing the value."""

    name: str
    r"""Name of the custom field."""

    organization_id: str
    r"""The ID of the organization owning the custom field."""

    properties: CustomFieldDateProperties

    TYPE: Annotated[
        Annotated[
            CustomFieldDateType,
            AfterValidator(validate_const(CustomFieldDateType.DATE)),
        ],
        pydantic.Field(alias="type"),
    ] = CustomFieldDateType.DATE

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = []
        nullable_fields = ["modified_at"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in self.model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .benefittype import BenefitType
from .listresource_benefit_ import ListResourceBenefit, ListResourceBenefitTypedDict
from polar_sdk.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from polar_sdk.utils import FieldMetadata, QueryParamMetadata
import pydantic
from pydantic import model_serializer
from typing import Callable, List, Optional, Union
from typing_extensions import Annotated, NotRequired, TypeAliasType, TypedDict


BenefitsListQueryParamOrganizationIDFilterTypedDict = TypeAliasType(
    "BenefitsListQueryParamOrganizationIDFilterTypedDict", Union[str, List[str]]
)
r"""Filter by organization ID."""


BenefitsListQueryParamOrganizationIDFilter = TypeAliasType(
    "BenefitsListQueryParamOrganizationIDFilter", Union[str, List[str]]
)
r"""Filter by organization ID."""


QueryParamBenefitTypeFilterTypedDict = TypeAliasType(
    "QueryParamBenefitTypeFilterTypedDict", Union[BenefitType, List[BenefitType]]
)
r"""Filter by benefit type."""


QueryParamBenefitTypeFilter = TypeAliasType(
    "QueryParamBenefitTypeFilter", Union[BenefitType, List[BenefitType]]
)
r"""Filter by benefit type."""


class BenefitsListRequestTypedDict(TypedDict):
    organization_id: NotRequired[
        Nullable[BenefitsListQueryParamOrganizationIDFilterTypedDict]
    ]
    r"""Filter by organization ID."""
    type_filter: NotRequired[Nullable[QueryParamBenefitTypeFilterTypedDict]]
    r"""Filter by benefit type."""
    page: NotRequired[int]
    r"""Page number, defaults to 1."""
    limit: NotRequired[int]
    r"""Size of a page, defaults to 10. Maximum is 100."""


class BenefitsListRequest(BaseModel):
    organization_id: Annotated[
        OptionalNullable[BenefitsListQueryParamOrganizationIDFilter],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filter by organization ID."""

    type_filter: Annotated[
        OptionalNullable[QueryParamBenefitTypeFilter],
        pydantic.Field(alias="type"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filter by benefit type."""

    page: Annotated[
        Optional[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = 1
    r"""Page number, defaults to 1."""

    limit: Annotated[
        Optional[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = 10
    r"""Size of a page, defaults to 10. Maximum is 100."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["organization_id", "type_filter", "page", "limit"]
        nullable_fields = ["organization_id", "type_filter"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in self.model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class BenefitsListResponseTypedDict(TypedDict):
    result: ListResourceBenefitTypedDict


class BenefitsListResponse(BaseModel):
    next: Callable[[], Optional[BenefitsListResponse]]

    result: ListResourceBenefit



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class ArticleVisibility(str, Enum):
    PUBLIC = "public"
    HIDDEN = "hidden"
    PRIVATE = "private"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .articlebyline import ArticleByline
from .articlevisibility import ArticleVisibility
from datetime import datetime
from polar_sdk.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from pydantic import model_serializer
from typing_extensions import NotRequired, TypedDict


class ArticleUpdateTypedDict(TypedDict):
    title: NotRequired[Nullable[str]]
    body: NotRequired[Nullable[str]]
    r"""Body in string format. body and body_base64 are mutually exclusive."""
    body_base64: NotRequired[Nullable[str]]
    r"""Body in base64-encoded format. Can be helpful to bypass Web Application Firewalls (WAF). body and body_base64 are mutually exclusive."""
    slug: NotRequired[Nullable[str]]
    byline: NotRequired[Nullable[ArticleByline]]
    r"""If the user or organization should be credited in the byline."""
    visibility: NotRequired[Nullable[ArticleVisibility]]
    paid_subscribers_only: NotRequired[Nullable[bool]]
    r"""Set to true to only make this article available for subscribers to a paid subscription tier in the organization."""
    paid_subscribers_only_ends_at: NotRequired[Nullable[datetime]]
    r"""If specified, time at which the article should no longer be restricted to paid subscribers. Only relevant if `paid_subscribers_only` is true."""
    published_at: NotRequired[Nullable[datetime]]
    r"""Time of publishing. If this date is in the future, the post will be scheduled to publish at this time."""
    notify_subscribers: NotRequired[Nullable[bool]]
    r"""Set to true to deliver this article via email and/or notifications to subscribers."""
    is_pinned: NotRequired[Nullable[bool]]
    r"""If the article should be pinned"""
    og_image_url: NotRequired[Nullable[str]]
    r"""Custom og:image URL value"""
    og_description: NotRequired[Nullable[str]]
    r"""Custom og:description value"""


class ArticleUpdate(BaseModel):
    title: OptionalNullable[str] = UNSET

    body: OptionalNullable[str] = UNSET
    r"""Body in string format. body and body_base64 are mutually exclusive."""

    body_base64: OptionalNullable[str] = UNSET
    r"""Body in base64-encoded format. Can be helpful to bypass Web Application Firewalls (WAF). body and body_base64 are mutually exclusive."""

    slug: OptionalNullable[str] = UNSET

    byline: OptionalNullable[ArticleByline] = UNSET
    r"""If the user or organization should be credited in the byline."""

    visibility: OptionalNullable[ArticleVisibility] = UNSET

    paid_subscribers_only: OptionalNullable[bool] = UNSET
    r"""Set to true to only make this article available for subscribers to a paid subscription tier in the organization."""

    paid_subscribers_only_ends_at: OptionalNullable[datetime] = UNSET
    r"""If specified, time at which the article should no longer be restricted to paid subscribers. Only relevant if `paid_subscribers_only` is true."""

    published_at: OptionalNullable[datetime] = UNSET
    r"""Time of publishing. If this date is in the future, the post will be scheduled to publish at this time."""

    notify_subscribers: OptionalNullable[bool] = UNSET
    r"""Set to true to deliver this article via email and/or notifications to subscribers."""

    is_pinned: OptionalNullable[bool] = UNSET
    r"""If the article should be pinned"""

    og_image_url: OptionalNullable[str] = UNSET
    r"""Custom og:image URL value"""

    og_description: OptionalNullable[str] = UNSET
    r"""Custom og:description value"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "title",
            "body",
            "body_base64",
            "slug",
            "byline",
            "visibility",
            "paid_subscribers_only",
            "paid_subscribers_only_ends_at",
            "published_at",
            "notify_subscribers",
            "is_pinned",
            "og_image_url",
            "og_description",
        ]
        nullable_fields = [
            "title",
            "body",
            "body_base64",
            "slug",
            "byline",
            "visibility",
            "paid_subscribers_only",
            "paid_subscribers_only_ends_at",
            "published_at",
            "notify_subscribers",
            "is_pinned",
            "og_image_url",
            "og_description",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in self.model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .checkout import Checkout, CheckoutTypedDict
from enum import Enum
from polar_sdk.types import BaseModel
from polar_sdk.utils import validate_const
import pydantic
from pydantic.functional_validators import AfterValidator
from typing_extensions import Annotated, TypedDict


class WebhookCheckoutUpdatedPayloadType(str, Enum):
    CHECKOUT_UPDATED = "checkout.updated"


class WebhookCheckoutUpdatedPayloadTypedDict(TypedDict):
    r"""Sent when a checkout is updated.

    **Discord & Slack support:** Basic
    """

    data: CheckoutTypedDict
    r"""Checkout session data retrieved using an access token."""
    type: WebhookCheckoutUpdatedPayloadType


class WebhookCheckoutUpdatedPayload(BaseModel):
    r"""Sent when a checkout is updated.

    **Discord & Slack support:** Basic
    """

    data: Checkout
    r"""Checkout session data retrieved using an access token."""

    TYPE: Annotated[
        Annotated[
            WebhookCheckoutUpdatedPayloadType,
            AfterValidator(
                validate_const(WebhookCheckoutUpdatedPayloadType.CHECKOUT_UPDATED)
            ),
        ],
        pydantic.Field(alias="type"),
    ] = WebhookCheckoutUpdatedPayloadType.CHECKOUT_UPDATED



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .subscription import Subscription, SubscriptionTypedDict
from enum import Enum
from polar_sdk.types import BaseModel
from polar_sdk.utils import validate_const
import pydantic
from pydantic.functional_validators import AfterValidator
from typing_extensions import Annotated, TypedDict


class WebhookSubscriptionActivePayloadType(str, Enum):
    SUBSCRIPTION_ACTIVE = "subscription.active"


class WebhookSubscriptionActivePayloadTypedDict(TypedDict):
    r"""Sent when a subscription becomes active,
    whether because it's a new paid subscription or because payment was recovered.

    **Discord & Slack support:** Full
    """

    data: SubscriptionTypedDict
    type: WebhookSubscriptionActivePayloadType


class WebhookSubscriptionActivePayload(BaseModel):
    r"""Sent when a subscription becomes active,
    whether because it's a new paid subscription or because payment was recovered.

    **Discord & Slack support:** Full
    """

    data: Subscription

    TYPE: Annotated[
        Annotated[
            WebhookSubscriptionActivePayloadType,
            AfterValidator(
                validate_const(WebhookSubscriptionActivePayloadType.SUBSCRIPTION_ACTIVE)
            ),
        ],
        pydantic.Field(alias="type"),
    ] = WebhookSubscriptionActivePayloadType.SUBSCRIPTION_ACTIVE



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from .sdkconfiguration import SDKConfiguration
from polar_sdk.downloadables import Downloadables
from polar_sdk.polar_advertisements import PolarAdvertisements
from polar_sdk.polar_benefits import PolarBenefits
from polar_sdk.polar_license_keys import PolarLicenseKeys
from polar_sdk.polar_orders import PolarOrders
from polar_sdk.polar_subscriptions import PolarSubscriptions


class Users(BaseSDK):
    benefits: PolarBenefits
    orders: PolarOrders
    subscriptions: PolarSubscriptions
    advertisements: PolarAdvertisements
    downloadables: Downloadables
    license_keys: PolarLicenseKeys

    def __init__(self, sdk_config: SDKConfiguration) -> None:
        BaseSDK.__init__(self, sdk_config)
        self.sdk_configuration = sdk_config
        self._init_sdks()

    def _init_sdks(self):
        self.benefits = PolarBenefits(self.sdk_configuration)
        self.orders = PolarOrders(self.sdk_configuration)
        self.subscriptions = PolarSubscriptions(self.sdk_configuration)
        self.advertisements = PolarAdvertisements(self.sdk_configuration)
        self.downloadables = Downloadables(self.sdk_configuration)
        self.license_keys = PolarLicenseKeys(self.sdk_configuration)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .benefitads import BenefitAds, BenefitAdsTypedDict
from .benefitarticles import BenefitArticles, BenefitArticlesTypedDict
from .benefitcustom import BenefitCustom, BenefitCustomTypedDict
from .benefitdiscord import BenefitDiscord, BenefitDiscordTypedDict
from .benefitdownloadables import BenefitDownloadables, BenefitDownloadablesTypedDict
from .benefitgithubrepository import (
    BenefitGitHubRepository,
    BenefitGitHubRepositoryTypedDict,
)
from .benefitlicensekeys import BenefitLicenseKeys, BenefitLicenseKeysTypedDict
from typing import Union
from typing_extensions import TypeAliasType


BenefitTypedDict = TypeAliasType(
    "BenefitTypedDict",
    Union[
        BenefitArticlesTypedDict,
        BenefitAdsTypedDict,
        BenefitDiscordTypedDict,
        BenefitGitHubRepositoryTypedDict,
        BenefitDownloadablesTypedDict,
        BenefitLicenseKeysTypedDict,
        BenefitCustomTypedDict,
    ],
)


Benefit = TypeAliasType(
    "Benefit",
    Union[
        BenefitArticles,
        BenefitAds,
        BenefitDiscord,
        BenefitGitHubRepository,
        BenefitDownloadables,
        BenefitLicenseKeys,
        BenefitCustom,
    ],
)
